#######################################################################
#
# $Id: Mesh.py,v 1.3 2008-02-28 05:05:01 lorenz Exp $
# 
# Copyright (C) 2004-2008 Petr Lorenz
# All rights reserved.
# 
# ##### BEGIN LICENSE BLOCK #####
#
# Version: MPL 1.1 / GPL 2.0 / LGPL 2.1
#
# The contents of this file are subject to the Mozilla Public License
# Version 1.1 (the "License"); you may not use this file except in
# compliance with the License. You may obtain a copy of the License at
# http://www.mozilla.org/MPL/
# 
# Software distributed under the License is distributed on an "AS IS"
# basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
# License for the specific language governing rights and limitations
# under the License.
# 
# The Original Code is Petr Lorenz emGine Graphical User Interface (GUI).
# 
# The Initial Developer of the Original Code is Petr Lorenz.
# Portions created by the Initial Developer are Copyright (C) 2004-2008.
# All Rights Reserved.
# 
# Alternatively, the contents of this file may be used under the terms of
# either the GNU General Public License Version 2 or later (the  "GPL"), or the
# GNU Lesser General Public License Version 2.1 or later (the "LGPL"), in which
# case the provisions of the GPL or the LGPL are applicable instead of those
# above.  If you wish to allow use of your version of this file only under the
# terms of either the GPL or the LGPL and not to allow others to use your
# version of this file under the MPL, indicate your decision by deleting the
# provisions above and replace them with the notice and other provisions
# required by the GPL or LGPL.  If you do not delete the provisions above, a
# recipient may use your version of this file under the terms of any one of the
# MPL License, the GPL or the LGPL.
#
# ##### END LICENSE BLOCK #####
#
#######################################################################
# 
# $Log: Mesh.py,v $
# Revision 1.3  2008-02-28 05:05:01  lorenz
# Zero rounding for too small distances in dl calculation.
#
# Revision 1.2  2008-02-07 05:45:25  lorenz
# * Fixed 'Simulation Boundary View' check problem
#   when closing a project.
#
# * Fixed Bounding Box problems when moving/editing a geometry.
#
# Revision 1.1.1.1  2008-01-28 06:15:35  lorenz
# Import emGine GUI v. 0.7.1
#
# Revision 1.5  2007-07-02 06:37:18  lorenz
# Clean up of old code.
#
# Revision 1.4  2007-06-14 06:31:51  lorenz
# Added emGine settings handling.
# Better handling of GUI state through GUIState class.
#
# Revision 1.3  2007-06-11 06:17:58  lorenz
# Construction plane handling added (during IMS2007, HAWAII + airplane)
#
# Revision 1.2  2007-05-18 16:37:01  lorenz
# Work on 'surrounding space' simbox and
#  'emgine_gui' tag
#
# Revision 1.1.1.1  2007-05-08 06:22:35  lorenz
# Initial Import
#
#######################################################################

# July 2006 - February 2008

import consts

# TODO: speed-up all computations here by using NumPy !!!! -PL-

class Mesh:
  def __init__(self):
    self.clear_all()

  def clear_all(self):
    self.x_points = list()
    self.y_points = list()
    self.z_points = list()
    self.simbox = [ [None, None, None], [None, None, None] ]
    self.dl = 0

  def add_xpoint(self, x):
    if not self.x_points.count(x):
      self.x_points.append(x)

  def add_ypoint(self, y):
    if not self.y_points.count(y):
      self.y_points.append(y)

  def add_zpoint(self, z):
    if not self.z_points.count(z):
      self.z_points.append(z)

  def add_point(self, point):
    [x, y, z] = point
    if not self.x_points.count(x):
      self.x_points.append(x)

    if not self.y_points.count(y):
      self.y_points.append(y)

    if not self.z_points.count(z):
      self.z_points.append(z)

  def add_points(self, points):
    for point in points:
      self.add_point(point)

  def set_dl(self, dl_):
    self.dl = dl_

  def get_dl(self):
    return self.dl

  def get_min_dl(self):
    if ((len(self.x_points) <= 1) and (len(self.y_points) <= 1) and (len(self.z_points) <= 1)):
      return None

    self.x_points.sort()
    self.y_points.sort()
    self.z_points.sort()

    dl = list()

    for i in range(1,len(self.x_points)):
      dl.append(self.x_points[i] - self.x_points[i-1])

    for i in range(1,len(self.y_points)):
      dl.append(self.y_points[i] - self.y_points[i-1])

    for i in range(1,len(self.z_points)):
      dl.append(self.z_points[i] - self.z_points[i-1])

    # return the smallest non-zero dl
    dl.sort()
    print dl
    ret_val = None
    for dl_ret in dl:
      if dl_ret >= consts.EMGINE_ROUND_ZERO_MESH_THRESHOLD:
	ret_val = dl_ret
	break
    return ret_val

  def get_bbox(self):
    if (len(self.x_points) == 0) or (len(self.y_points) == 0) or (len(self.z_points) == 0):
      if self.simbox != [[None, None, None], [None, None, None]]:
	return self.simbox
      else:
	return ((None, None, None), (None, None, None))

    self.x_points.sort()
    self.y_points.sort()
    self.z_points.sort()

    p1 = [self.x_points[0], self.y_points[0], self.z_points[0]]
    p2 = [self.x_points[-1], self.y_points[-1], self.z_points[-1]]

    # test if simbox is not greater
    if self.simbox:
      for ii in range(3):
	if self.simbox[0][ii] != None:
	  if self.simbox[0][ii] < p1[ii]:
	    p1[ii] = self.simbox[0][ii]

      for ii in range(3):
	if self.simbox[1][ii] != None:
	  if self.simbox[1][ii] > p2[ii]:
	    p2[ii] = self.simbox[1][ii]
    
    return (p1, p2)

  def set_simbox(self, simbox_):
    #TODO: no checkings on data done here ! Do it !
    # FIXME: How to do a deep copy ?
    p1 = [simbox_[0][0], simbox_[0][1], simbox_[0][2]]
    p2 = [simbox_[1][0], simbox_[1][1], simbox_[1][2]]
    self.simbox = [p1, p2]

  def set_simbox_xmin(self, xmin_):
    self.simbox[0][0] = xmin_

  def set_simbox_xmax(self, xmax_):
    self.simbox[1][0] = xmax_

  def set_simbox_ymin(self, ymin_):
    self.simbox[0][1] = ymin_

  def set_simbox_ymax(self, ymax_):
    self.simbox[1][1] = ymax_

  def set_simbox_zmin(self, zmin_):
    self.simbox[0][2] = zmin_

  def set_simbox_zmax(self, zmax_):
    self.simbox[1][2] = zmax_

###############################################
if __name__ == '__main__':
  from GroupGeometryObjects import *
  from GeometryObjectPatch import *

  print "Testing Mesh Module..."

  mesh = Mesh()

  mesh.get_bbox()
  mesh.add_xpoint(1)
  print mesh.get_bbox()
  mesh.add_xpoint(-1)
  print mesh.get_bbox()
  mesh.add_ypoint(2)
  print mesh.get_bbox()
  mesh.add_zpoint(2)
  print mesh.get_bbox()
  mesh.add_point([0,2,2])
  print mesh.get_bbox()
  mesh.add_point([-4,-4,-4])
  print mesh.get_bbox()

  mesh.add_points(([-11,-11,-11],[-12,-12,-12],[111,111,111]))
  print mesh.get_bbox()

  #######################
  print "---- Testing with GroupGeometryObjects ----"
  group = GroupGeometryObjects()
  patch1 = GeometryObjectPatch("Patch1", [0,0,0], [1,1,0])
  patch2 = GeometryObjectPatch("Patch2", [-1,-1,-1], [0.5,0.5,-1])
  group.add(patch1)
  group.add(patch2)
  dpoints = group.get_discretization_points()
  mesh2 = Mesh()
  mesh2.add_points(dpoints)
  print dpoints
  print mesh2.get_bbox()
  print mesh2.get_min_dl()

  #######################
  print "---- Testing Simbox ----"
  print mesh2.get_bbox()
  mesh2.set_simbox_zmax(11)
  print mesh2.get_bbox()



#!/usr/bin/env python
#######################################################################
#
# $Id: emGine.py,v 1.3 2008-02-07 05:45:15 lorenz Exp $
#
# Copyright (C) 2004-2008 Petr Lorenz
# All rights reserved.
#
# ##### BEGIN LICENSE BLOCK #####
#
# Version: MPL 1.1 / GPL 2.0 / LGPL 2.1
#
# The contents of this file are subject to the Mozilla Public License
# Version 1.1 (the "License"); you may not use this file except in
# compliance with the License. You may obtain a copy of the License at
# http://www.mozilla.org/MPL/
# 
# Software distributed under the License is distributed on an "AS IS"
# basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
# License for the specific language governing rights and limitations
# under the License.
# 
# The Original Code is Petr Lorenz emGine Graphical User Interface (GUI).
# 
# The Initial Developer of the Original Code is Petr Lorenz.
# Portions created by the Initial Developer are Copyright (C) 2004-2008.
# All Rights Reserved.
# 
# Alternatively, the contents of this file may be used under the terms of
# either the GNU General Public License Version 2 or later (the  "GPL"), or the
# GNU Lesser General Public License Version 2.1 or later (the "LGPL"), in which
# case the provisions of the GPL or the LGPL are applicable instead of those
# above.  If you wish to allow use of your version of this file only under the
# terms of either the GPL or the LGPL and not to allow others to use your
# version of this file under the MPL, indicate your decision by deleting the
# provisions above and replace them with the notice and other provisions
# required by the GPL or LGPL.  If you do not delete the provisions above, a
# recipient may use your version of this file under the terms of any one of the
# MPL License, the GPL or the LGPL.
#
# ##### END LICENSE BLOCK #####
#
#######################################################################
# 
# $Log: emGine.py,v $
# Revision 1.3  2008-02-07 05:45:15  lorenz
# * Fixed 'Simulation Boundary View' check problem
#   when closing a project.
#
# * Fixed Bounding Box problems when moving/editing a geometry.
#
# Revision 1.2  2008-02-01 14:20:06  lorenz
# Relative path set to '.' when empty.
#
# Revision 1.1.1.1  2008-01-28 06:15:35  lorenz
# Import emGine GUI v. 0.7.1
#
# Revision 1.5  2007-07-02 06:36:59  lorenz
# Clean up of old code.
#
# Revision 1.4  2007-06-14 06:31:00  lorenz
# Added emGine settings
#
# Revision 1.3  2007-06-11 06:16:30  lorenz
# Added blank start
#
# Revision 1.2  2007-05-10 16:29:37  lorenz
# Changes in Copyright notice.
#
# Revision 1.1.1.1  2007-05-08 06:22:35  lorenz
# Initial Import
#
#######################################################################
#
from xml.dom.minidom import parse, parseString, Node

import sys, os
from emGineGUI import MainWindow


##############################
if __name__ == '__main__':
  #
  # quick & Dirty hack to avoid Segfault, due to
  # uninitialized xml.dom.minidom
  #
  # This segmentation fault occurs when vtkTkRenderWidget
  # is used on some configurations (e.g., Slackware Linux 10.1)!!!
  #
  # 13.02.2006, Petr Lorenz
  #
  dom_dummy = parseString("<dummy/>")

  print "emGine GUI, Copyright (C) 2004-2008 Petr Lorenz"
  print "All rights reserved."
  print ""

  print "This software is distributed on an \"AS IS\" basis,"
  print "WITHOUT WARRANTY OF ANY KIND, either express or implied."
  print "For details, see the LICENSE file."
  
  print ""
  print "Starting emGine GUI..."

  # Here we go, really
  args = sys.argv

  # change the working directory to the app directory
  rel_path = os.path.split(args[0])[0]
  if rel_path == "":
    rel_path = "."
  os.chdir(rel_path)

  main_window = MainWindow.MainWindow()

  if len(args) > 1:
    main_window.open_project_from_file(args[1])

  main_window.MainLoop()



#######################################################################
#
# $Id: consts.py,v 1.11 2008-04-07 05:09:35 lorenz Exp $
# 
# Copyright (C) 2004-2008 Petr Lorenz
# All rights reserved.
# 
# ##### BEGIN LICENSE BLOCK #####
#
# Version: MPL 1.1 / GPL 2.0 / LGPL 2.1
#
# The contents of this file are subject to the Mozilla Public License
# Version 1.1 (the "License"); you may not use this file except in
# compliance with the License. You may obtain a copy of the License at
# http://www.mozilla.org/MPL/
# 
# Software distributed under the License is distributed on an "AS IS"
# basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
# License for the specific language governing rights and limitations
# under the License.
# 
# The Original Code is Petr Lorenz emGine Graphical User Interface (GUI).
# 
# The Initial Developer of the Original Code is Petr Lorenz.
# Portions created by the Initial Developer are Copyright (C) 2004-2008.
# All Rights Reserved.
# 
# Contributor(s): Bruno Biscontini, Rachid Khlifi.
# 
# Alternatively, the contents of this file may be used under the terms of
# either the GNU General Public License Version 2 or later (the  "GPL"), or the
# GNU Lesser General Public License Version 2.1 or later (the "LGPL"), in which
# case the provisions of the GPL or the LGPL are applicable instead of those
# above.  If you wish to allow use of your version of this file only under the
# terms of either the GPL or the LGPL and not to allow others to use your
# version of this file under the MPL, indicate your decision by deleting the
# provisions above and replace them with the notice and other provisions
# required by the GPL or LGPL.  If you do not delete the provisions above, a
# recipient may use your version of this file under the terms of any one of the
# MPL License, the GPL or the LGPL.
#
# ##### END LICENSE BLOCK #####
#
#######################################################################
# 
# $Log: consts.py,v $
# Revision 1.11  2008-04-07 05:09:35  lorenz
# Smith diagram integration.
#
# Revision 1.10  2008-03-28 06:25:37  lorenz
# - Perspective/Orthogonal view
# - Top/Bottom/Front/Back/Left/Right views
# - Boundary conditions setup
# - Platform Info
#
# Revision 1.9  2008-02-28 05:05:01  lorenz
# Zero rounding for too small distances in dl calculation.
#
# Revision 1.8  2008-02-14 05:59:26  lorenz
# Simulation thread improvements - lock, better IPC, termination.
#
# Revision 1.7  2008-02-13 15:28:27  lorenz
# Windows simulation termination implemented.
#
# Revision 1.6  2008-02-08 05:04:57  lorenz
# Unix support for terminating a simulation.
#
# Revision 1.5  2008-02-07 05:45:25  lorenz
# * Fixed 'Simulation Boundary View' check problem
#   when closing a project.
#
# * Fixed Bounding Box problems when moving/editing a geometry.
#
# Revision 1.4  2008-02-03 16:34:34  lorenz
# Version information update.
#
# Revision 1.3  2008-01-30 05:29:04  lorenz
# Application Icon added
#
# Revision 1.2  2008-01-28 16:15:15  lorenz
# Windows related changes (e.g., refreshing).
#
# Revision 1.1.1.1  2008-01-28 06:15:35  lorenz
# Import emGine GUI v. 0.7.1
#
# Revision 1.6  2007-07-02 06:37:18  lorenz
# Clean up of old code.
#
# Revision 1.5  2007-06-28 05:34:28  lorenz
# Checkbutton in 'View->Simulation Boundary' added.
#
# Preliminary support for polygons added.
#
# Revision 1.4  2007-06-18 03:55:36  lorenz
# Status bar extended
#
# Revision 1.3  2007-05-18 16:37:01  lorenz
# Work on 'surrounding space' simbox and
#  'emgine_gui' tag
#
# Revision 1.2  2007-05-14 04:53:51  lorenz
# New code for multiport time-domain signals and S-parameters
#
# Revision 1.1.1.1  2007-05-08 06:22:35  lorenz
# Initial Import
#
#######################################################################

#--------------------------------------------------
# general information
#--------------------------------------------------
REVISION = "$Revision: 1.11 $"
LAST_UPDATE = "01.04.2008"
VERSION = "0.7.4" + " - " + LAST_UPDATE


#--------------------------------------------------
# The Main Window Status
#--------------------------------------------------
MW_TITLE = "%s %s" %("emGine GUI, v", VERSION)

#--------------------------------------------------
# Simulation related constants
#--------------------------------------------------
EMGINE_DEEMBEDDING_CELLS = 20
EMGINE_LARGE_NUMBER_OF_CELLS = 10e6

EMGINE_SIMULATION_RUNNING = 150

EMGINE_SIMULATION_FINISHED = 200
EMGINE_SIMULATION_FINISHED_OK = 201
EMGINE_SIMULATION_FINISHED_ERROR = 202
EMGINE_SIMULATION_EXIT_THREAD = 210

EMGINE_SIMULATION_TERMINATE = 300

#--------------------------------------------------
# Default material if no material chosen
#--------------------------------------------------
EMGINE_MATERIAL_FREE_SPACE = 'Air'
EMGINE_MATERIAL_PEC = 'PEC'
EMGINE_MATERIAL_PMC = 'PMC'
EMGINE_DEFAULT_MATERIAL = EMGINE_MATERIAL_FREE_SPACE

#--------------------------------------------------
# Rounding errors related constants
#--------------------------------------------------
EMGINE_ROUND_ZERO_MESH_THRESHOLD = 1e-10

#--------------------------------------------------
# STATEs of the program
#--------------------------------------------------
STATE_NONE = 0

STATE_INITIALIZED = 1

STATE_PROJECT_CHANGED = 10
#STATE_PROJECT_NEW = 2
#STATE_PROJECT_OPENED = 3
#STATE_PROJECT_CLOSED = 4
#STATE_PROJECT_SAVED = 5
#STATE_PROJECT_MODIFIED = 6

STATE_TREE_CLICKED_PROJECT = 100
STATE_TREE_CLICKED_FOLDER = 105
STATE_TREE_CLICKED_SFOLDER = 110
STATE_TREE_CLICKED_SIMULATION = 115
STATE_TREE_CLICKED_FILE = 120

STATE_TREE_CLICKED_GEOMETRY = 200
STATE_TREE_CLICKED_GEOMETRY_NODE = 201

STATE_TREE_CLICKED_EXCITATION = 300
STATE_TREE_CLICKED_EXCITATION_NODE = 301

STATE_TREE_CLICKED_MATERIAL = 400

STATE_TREE_CLICKED_RESULTS = 500
STATE_TREE_CLICKED_RESULTS2D = 510
STATE_TREE_CLICKED_PLOT_SMITH = 511
STATE_TREE_CLICKED_RESULTS2D_ALL_TDSIGNALS = 515
STATE_TREE_CLICKED_RESULTS2D_ALL_SPARAMETERS = 516
STATE_TREE_CLICKED_PLOT_ALL_SMITH = 517

#--------------------------------------------------
# MISC
#--------------------------------------------------
GUI_XML_NODE = "GUI__xml_node"

#--------------------------------------------------
# PROJECT STATUS
#--------------------------------------------------
PRJ_STATE_INITIALIZED = 1
PRJ_STATE_OPENED = 2
PRJ_STATE_NEW = 3
PRJ_STATE_CLOSED = 4
PRJ_STATE_SAVED = 5
PRJ_STATE_MODIFIED = 6

PRJ_DEFAULT_FNAME = "Noname.emm"
PRJ_TLMGUI_VERSION = "0.1"

PRJ_TAG_PROJECT = "project"
PRJ_TAG_FOLDER = "folder"
PRJ_TAG_SFOLDER = "sfolder"
PRJ_TAG_SIMULATION = "simulation"
PRJ_TAG_FILE = "file"

#--------------------------------------------------
# XML Tags
#--------------------------------------------------

# emGine specific
XML_TAG_EMGINE_GUI = "emgine_gui"
XML_TAG_SURROUNDING_SPACE = "surrounding_space"

#
XML_TAG_PROJECT = "project"

XML_TAG_MATERIALS = "materials"
XML_TAG_MATERIAL = "material"
XML_TAG_EPSR = "eps_r"
XML_TAG_MUR = "mu_r"
XML_TAG_CONDUCTIVITY = "conductivity"
XML_TAG_TAND = "tand"

XML_TAG_GEOMETRY = "geometry"
XML_TAG_EXCITATION = "excitation"

# Boundaries
XML_TAG_BOUNDARIES = "boundaries"
XML_TAG_BOUNDARY = "boundary"

XML_TAG_BOUNDARY_XMIN = "xmin"
XML_TAG_BOUNDARY_XMAX = "xmax"
XML_TAG_BOUNDARY_YMIN = "ymin"
XML_TAG_BOUNDARY_YMAX = "ymax"
XML_TAG_BOUNDARY_ZMIN = "zmin"
XML_TAG_BOUNDARY_ZMAX = "zmax"

# Boundary conditions
XML_TAG_BC = "boundary_condition"
XML_BC_ATTR_TYPE = "type"
XML_BC_TYPE_PEC = "PEC"
XML_BC_TYPE_PMC = "PMC"
XML_BC_TYPE_ABSORBING = "absorbing"

XML_TAG_X = "x"
XML_TAG_Y = "y"
XML_TAG_Z = "z"

XML_TAG_VECTOR2D_COMPONENTS = [XML_TAG_X, XML_TAG_Y]
XML_TAG_VECTOR3D_COMPONENTS = [XML_TAG_X, XML_TAG_Y, XML_TAG_Z]

XML_TAG_POINT2D_COMPONENTS = XML_TAG_VECTOR2D_COMPONENTS
XML_TAG_POINT3D_COMPONENTS = XML_TAG_VECTOR3D_COMPONENTS

XML_TAG_X1 = "x1"
XML_TAG_Y1 = "y1"
XML_TAG_Z1 = "z1"
XML_TAG_X2 = "x2"
XML_TAG_Y2 = "y2"
XML_TAG_Z2 = "z2"
XML_TAG_LENGTH_X = "length_x"
XML_TAG_LENGTH_Y = "length_y"
XML_TAG_LENGTH_Z = "length_z"


# Geomtry tags
XML_TAG_PATCH = "patch"

XML_TAG_CIRCLE = "circle"

XML_TAG_CUBE = "cube"

XML_TAG_CYLINDER = "cylinder"
XML_TAG_CYLINDER_ORIENTATION = "orientation"
XML_TAG_RADIUS = "radius"
XML_TAG_HEIGHT = "height"

XML_TAG_POINT2D = "p2d"
XML_TAG_POINT3D = "p3d"
XML_TAG_VECTOR2D = "v2d"
XML_TAG_VECTOR3D = "v3d"
XML_TAG_POLYGON2D = "polygon2d"

XML_TAG_NORMAL = "normal"
XML_TAG_LEVEL = "level"
XML_TAG_POINTS = "points"



XML_GEOMETRY_GROUP = [XML_TAG_PATCH,
                      XML_TAG_CIRCLE,
		      XML_TAG_POLYGON2D,
                      XML_TAG_CUBE,
                      XML_TAG_CYLINDER]



# Excitation tags
XML_TAG_VOLTAGE_PORT = "voltage_port"

XML_EXCITATION_GROUP = [XML_TAG_VOLTAGE_PORT]

# signals
XML_TAG_SIGNALS = "signals"
XML_TAG_SIGNAL_GAUSSIAN_DIFFERENTIATED = "gaussian_differentiated"
XML_TAG_SIGNAL_GAUSSIAN_MODULATED = "gaussian_modulated"
XML_TAG_FREQUENCY_MIN = "frequency_min"
XML_TAG_FREQUENCY_MAX = "frequency_max"
XML_TAG_TSHIFT = "tshift"

# simulation
XML_TAG_SIMULATION = "simulation"
XML_TAG_SIMULATION_TLM = "tlm"
XML_TAG_NUMBER_OF_TIME_STEPS = "number_of_time_steps"

# mesh
XML_TAG_MESH = "mesh"
XML_TAG_DX = "dx"
XML_TAG_DY = "dy"
XML_TAG_DZ = "dz"

# results
XML_TAG_RESULTS = "results"
XML_TAG_RESULT = "result"
XML_ATTR_RESULT_TYPE = "type"

XML_TAG_TDSIGNAL = "tdsignal"
XML_TAG_SPARAMETERS = "sparameters"
XML_TAG_ZPARAMETERS = "zparameters"

#XML_TAG_TDSIGNALS = "tdsignals"

RESULT_TYPE_SPARAMETERS = "SParameters"
RESULT_TYPE_IMPEDANCE = "Impedance"
RESULT_TYPE_TIME_DOMAIN_SIGNAL = "TimeDomainSignal"

# emgine_result
XML_TAG_EMGINE_RESULT = "emgine_result"
RESULT_TYPE_SIMULATION = "simulation"
RESULT_TYPE_REFERENCE = "reference"

# Files
XML_TAG_FILE = "file"
XML_ATTR_FILE_SRC = "src"
XML_ATTR_FILE_COLUMN = "column"
XML_ATTR_FILE_TYPE = "type"

FILE_TYPE_ASCII = "ascii"
FILE_TYPE_S1P = "s1p"
FILE_TYPE_S2P = "s2p"
FILE_TYPE_V5D = "v5d"
FILE_TYPE_HTML = "html"

# Parameters
XML_TAG_PARAMETERS = "parameters"
XML_TAG_PARAMETER = "parameter"

# Units
XML_TAG_UNITS = "units"
XML_TAG_UNITS_LENGTH = "length"
XML_TAG_UNITS_FREQUENCY = "frequency"
XML_TAG_UNITS_TIME = "time"

# Misc tags
XML_TAG_INFINITE = "INFINITE"
XML_TAG_ORIENTATION_X = "X"
XML_TAG_ORIENTATION_Y = "Y"
XML_TAG_ORIENTATION_Z = "Z"

# Attributes
XML_ATTR_NAME = "name"
XML_ATTR_GEOMETRY_MATERIAL = "material"
XML_ATTR_UNITS = "units"

XML_ATTR_PARAMETER = "parameter"

XML_ATTR_NORMAL = "normal"
XML_ATTR_LEVEL = "level"

# Groups
XML_RESULTS2D_GROUP = [XML_TAG_TDSIGNAL,
		       XML_TAG_SPARAMETERS,
		       XML_TAG_ZPARAMETERS]
                       
		       


### DEFAULT_MATERIALS = """
### <materials>
### <material name=\"Air\">
###   <eps_r>1.0</eps_r>
###   <mu_r>1.0</mu_r>
### </material>
### <material name=\"PEC\">
###   <eps_r>1.0</eps_r>
###   <mu_r>1.0</mu_r>
###   <conductivity><INFINITE/></conductivity>
### </material>
### <material name=\"Copper\">
###   <eps_r>1.0</eps_r>
###   <mu_r>1.0</mu_r>
###   <conductivity>5.8e7</conductivity>
### </material>
### </materials>
### """

DEFAULT_UNITS = """
<units>
  <length>1</length>
  <frequency>1</frequency>
  <time>1</time>
</units>
"""

DEFAULT_MATERIALS = """
<materials>
<material name=\"Air\">
  <eps_r>1.0</eps_r>
  <mu_r>1.0</mu_r>
</material>
<material name=\"PEC\">
  <eps_r>1.0</eps_r>
  <mu_r>1.0</mu_r>
  <conductivity><INFINITE/></conductivity>
</material>
</materials>
"""

DEFAULT_GEOMETRY = """
<geometry>
</geometry>
"""

DEFAULT_SIGNALS = """
<signals>
  <gaussian_differentiated>
    <frequency_min>1e9</frequency_min>
    <frequency_max>3e9</frequency_max>
  </gaussian_differentiated>
</signals>
"""

DEFAULT_EXCITATION = """
<excitation>
</excitation>
"""

#DEFAULT_BOUNDARY = """
#<boundaries>
#<boundary name=\"Magnetic wall\">
#  <ref>1.0</ref>
#</boundary>
#<boundary name=\"Electric wall\">
#  <ref>-1.0</ref>
#</boundary>
#<boundary name=\"Absorbing wall\">
#  <ref>0.0</ref>
#</boundary>
#</boundaries>
#"""

DEFAULT_BOUNDARY = """
<boundaries>
</boundaries>
"""

DEFAULT_SIMULATION = """
<simulation>
  <tlm>
    <number_of_time_steps>100</number_of_time_steps>
    <mesh type="cartesian" mode="regular">
      <x1>-14</x1>
      <y1>-10</y1>
      <z1>-4</z1>
      <x2>14</x2>
      <y2>10</y2>
      <z2>4</z2>
      <dx>.1</dx>
      <dy>.1</dy>
      <dz>.1</dz>
    </mesh>
  </tlm>
</simulation>
"""

DEFAULT_RESULTS = """
<results>
</results>
"""

DEFAULT_PROJECT = "%s %s %s %s %s %s %s %s" %(DEFAULT_UNITS, DEFAULT_MATERIALS, DEFAULT_GEOMETRY, DEFAULT_SIGNALS, DEFAULT_EXCITATION, DEFAULT_BOUNDARY, DEFAULT_SIMULATION, DEFAULT_RESULTS)



#######################################################################
#
# $Id: PlotSmith.py,v 1.2 2008-04-07 05:09:35 lorenz Exp $
# 
# Copyright (C) 2004-2008 Petr Lorenz
# All rights reserved.
# 
# ##### BEGIN LICENSE BLOCK #####
#
# Version: MPL 1.1 / GPL 2.0 / LGPL 2.1
#
# The contents of this file are subject to the Mozilla Public License
# Version 1.1 (the "License"); you may not use this file except in
# compliance with the License. You may obtain a copy of the License at
# http://www.mozilla.org/MPL/
# 
# Software distributed under the License is distributed on an "AS IS"
# basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
# License for the specific language governing rights and limitations
# under the License.
# 
# The Original Code is Petr Lorenz emGine Graphical User Interface (GUI).
# 
# The Initial Developer of the Original Code is Petr Lorenz.
# Portions created by the Initial Developer are Copyright (C) 2004-2008.
# All Rights Reserved.
#
# Contributor(s): Sherif Sayed Ahmed
# 
# Alternatively, the contents of this file may be used under the terms of
# either the GNU General Public License Version 2 or later (the  "GPL"), or the
# GNU Lesser General Public License Version 2.1 or later (the "LGPL"), in which
# case the provisions of the GPL or the LGPL are applicable instead of those
# above.  If you wish to allow use of your version of this file only under the
# terms of either the GPL or the LGPL and not to allow others to use your
# version of this file under the MPL, indicate your decision by deleting the
# provisions above and replace them with the notice and other provisions
# required by the GPL or LGPL.  If you do not delete the provisions above, a
# recipient may use your version of this file under the terms of any one of the
# MPL License, the GPL or the LGPL.
#
# ##### END LICENSE BLOCK #####
#
#######################################################################
# 
# $Log: PlotSmith.py,v $
# Revision 1.2  2008-04-07 05:09:35  lorenz
# Smith diagram integration.
#
# Revision 1.1  2008-03-28 06:18:27  lorenz
# Initial commit.
#
#
#######################################################################

#
# March 2008
#
# 2D plotting with matplotlib
#
import wx

from matplotlib.backends.backend_wxagg import FigureCanvasWxAgg as FigureCanvas
from matplotlib.backends.backend_wx import NavigationToolbar2Wx
from matplotlib.figure import Figure

##import XYDataSource

# to calculate the phases
from numpy import *

##def phase(num):
##    return math.atan2(num.imag, num.real)
##  
class PlotSmith:
  def __init__(self, master):

    self.hold = 0
    
    # create the matplotlib-based Plot
    self.figure = Figure(facecolor = 'w')
  
    self.figure_canvas = FigureCanvas(master, -1, self.figure)
    self.hide()		# hide the widget

  def clear(self):
    self.figure.clf()

  def hold_on(self):
    self.hold = True
    self.axes.hold(True)

  def hold_off(self):
    self.hold = False
    self.axes.hold(False)

  def set_dimensions(self, x1, y1, x2, y2):
    self.figure_canvas.SetDimensions(x1, y1, x2, y2)

  def show(self):
    self.figure_canvas.Show(True)

  def hide(self):
    self.figure_canvas.Show(False)

  def set_legend(self, legend):
    self.axes.set_title(legend, fontsize =20)

  def set_legend_01(self, legend = None):
    #ToDo: autoscale the figure windows to allow good view for the legend box
    
##    self.axes.legend(legend)
    self.axes.legend(handlelen=0.05, loc=(1.06, 0.85), shadow=True)
##    self.axes.autoscale_view


  def plot_data(self,  real_values, imag_values , freq_values, title = "No data name"):
    
    if (len(real_values) != len(imag_values)):
      print "!! Error! Something is wrong with your data to plot"
      return

    if (len(real_values) != len(freq_values)):
      print "!! Error! Something is wrong with your data to plot"
      return

    # convert to numpy array
    data = array(real_values, dtype=complex) 
    data.imag = imag_values
    
    if not self.hold:
      self.figure.clf()
      self.axes = self.figure.add_axes([0.1,0.1,0.8,0.8], polar=True, aspect='equal')
      # plot smithchart circles
      # -----------------------
      self.plotsmithcircles()
    
    # Plot the data
    #--------------    
    self.axes.set_rmax(1)
    self.axes.plot(angle(data),abs(data),label=title,linewidth = 2,zorder=10)
    #self.axes.set_title( "Smith chart" , fontsize =20)
   
    # Show freq limits
    #-----------------
    self.axes.plot([angle(data[0])],[abs(data[0])],'oc',linewidth = 5,zorder=10)
    self.axes.plot([angle(data[len(data)-1])],[abs(data[len(data)-1])],'sm',linewidth = 5,zorder=10)

    # Disable Polar grid
    #------------------- 
    self.axes.set_rgrids([.001,999],['',''])
    self.axes.set_thetagrids([0,99],['',''])
    self.axes.grid(False)
    
  def plotsmithcircles(self):
    linetype = 'k'
    linewidth = 0.5
    linewidth2 = 1.5

    # plot R circle  
    r_array = (0.0, 0.2, 0.4, 0.7, 1.0, 1.4, 2.0, 3.0, 5.0)
    z = zeros((1,2000),complex)
    z.imag = arange(-50,50,0.05) 
    x_shift = 1e-3 # can be 1e-3 but not zero
    for r in r_array:
      z.real = r
      gamma_r = transpose((z-1)/(z+1))
      
      if r==1:
	self.axes.plot(angle(gamma_r), abs(gamma_r), linetype,linewidth = linewidth2,zorder=0)
      else:
	self.axes.plot(angle(gamma_r), abs(gamma_r), linetype,linewidth = linewidth,zorder=0)
      
      if not r==0:
	self.axes.text(angle((r+x_shift*1j-1)/(r+x_shift*1j+1)), 1e-6+abs((r+x_shift*1j-1)/(r+x_shift*1j+1)), "%.1f" %(r), backgroundcolor='white', alpha=1, horizontalalignment='center', verticalalignment='center', rotation=0)      
            
    # plot X circle   
    x_array = (0.2, 0.4, 0.6, 0.8, 1.0, 1.4, 2.0, 3.0, 5.0, 10.0)
    z = zeros((1,1000),complex)
    z.real = arange(0, 100, 0.1)
    for x in x_array:
      z.imag = x
      gamma_x = transpose((z-1)/(z+1))
      
      if x==1:
	self.axes.plot(angle(gamma_x),abs(gamma_x), linetype,linewidth = linewidth2,zorder=0)
      else:
	self.axes.plot(angle(gamma_x),abs(gamma_x), linetype,linewidth = linewidth,zorder=0)
	  
	  
      self.axes.text(angle((x*1j-1)/(x*1j+1)),1.05,"%.1f" %(x),horizontalalignment='center',verticalalignment='center',rotation=angle((x*1j-1)/(x*1j+1),'Degrees')-90)
        
    for x in multiply(-1,x_array):
      z.imag = x
      gamma_x = transpose((z-1)/(z+1))
      
      if x==-1:
	self.axes.plot(angle(gamma_x),abs(gamma_x), linetype,linewidth = linewidth2,zorder=0)
      else:
	self.axes.plot(angle(gamma_x),abs(gamma_x), linetype,linewidth = linewidth,zorder=0)
	  
      self.axes.text(angle((x*1j-1)/(x*1j+1)),1.05,"%.1f" %(x),horizontalalignment='center',verticalalignment='center',rotation=angle((x*1j-1)/(x*1j+1),'Degrees')+90)
 
    # plot X=0 line
    self.axes.plot([-pi , 0],[1 , 1], linetype,linewidth = linewidth2 ,zorder=0)
    self.axes.text(-pi ,1.0,'0',horizontalalignment='center',verticalalignment='center',rotation = 0 , backgroundcolor='white',fontsize=12)
 
    # plot infinite symbol
    self.axes.text(0 ,1.0,'$\infty$',horizontalalignment='center',verticalalignment='center',rotation = 0 , backgroundcolor='white',fontsize=12)

  def plot_dummy(self):

    r = transpose(.1+arange ( 0 , 0.7 , 0.001))
    theta = -4.5 * pi *r
    freq = r*10e9
    data = multiply(r,exp(1j*theta))

    self.plot_data(data.real, data.imag,freq,'my first data')
    
    self.hold_on()


    r = transpose(arange ( 0 , 0.9 , 0.001))
    theta = +2 * pi *r
    freq = r*10e9
    data = multiply(r,exp(1j*theta))

    self.plot_data(data.real, data.imag,freq,'my second data')
    
    self.set_legend("Smith Chart Plot")
    self.set_legend_01()

###############################################
if __name__ == '__main__':
  print "Testing PlotSmith Module..."


  app = wx.PySimpleApp()
  frame = wx.Frame(None, -1, "Testing PlotSmith Module", size = (800, 600))
  frame.Show(True)

  PlotSmith = PlotSmith(frame)
  PlotSmith.plot_dummy()
  PlotSmith.show()

  app.MainLoop()

  app.Destroy() 




#######################################################################
#
# $Id: Solver.py,v 1.2 2008-02-07 05:45:25 lorenz Exp $
# 
# Copyright (C) 2004-2008 Petr Lorenz
# All rights reserved.
# 
# ##### BEGIN LICENSE BLOCK #####
#
# Version: MPL 1.1 / GPL 2.0 / LGPL 2.1
#
# The contents of this file are subject to the Mozilla Public License
# Version 1.1 (the "License"); you may not use this file except in
# compliance with the License. You may obtain a copy of the License at
# http://www.mozilla.org/MPL/
# 
# Software distributed under the License is distributed on an "AS IS"
# basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
# License for the specific language governing rights and limitations
# under the License.
# 
# The Original Code is Petr Lorenz emGine Graphical User Interface (GUI).
# 
# The Initial Developer of the Original Code is Petr Lorenz.
# Portions created by the Initial Developer are Copyright (C) 2004-2008.
# All Rights Reserved.
# 
# Alternatively, the contents of this file may be used under the terms of
# either the GNU General Public License Version 2 or later (the  "GPL"), or the
# GNU Lesser General Public License Version 2.1 or later (the "LGPL"), in which
# case the provisions of the GPL or the LGPL are applicable instead of those
# above.  If you wish to allow use of your version of this file only under the
# terms of either the GPL or the LGPL and not to allow others to use your
# version of this file under the MPL, indicate your decision by deleting the
# provisions above and replace them with the notice and other provisions
# required by the GPL or LGPL.  If you do not delete the provisions above, a
# recipient may use your version of this file under the terms of any one of the
# MPL License, the GPL or the LGPL.
#
# ##### END LICENSE BLOCK #####
#
#######################################################################
# 
# $Log: Solver.py,v $
# Revision 1.2  2008-02-07 05:45:25  lorenz
# * Fixed 'Simulation Boundary View' check problem
#   when closing a project.
#
# * Fixed Bounding Box problems when moving/editing a geometry.
#
# Revision 1.1.1.1  2008-01-28 06:15:35  lorenz
# Import emGine GUI v. 0.7.1
#
# Revision 1.2  2007-07-02 06:37:18  lorenz
# Clean up of old code.
#
# Revision 1.1.1.1  2007-05-08 06:22:35  lorenz
# Initial Import
#
#######################################################################

from tlmObject import tlmObject

class Solver(tlmObject):
  def __init__(self, type_ = "Solver"):
    tlmObject.__init__(self, type_)

###############################################
if __name__ == '__main__':
  print "Testing Solver Module..."



#######################################################################
#
# $Id: ResultImpedance.py,v 1.2 2008-02-07 05:45:25 lorenz Exp $
# 
# Copyright (C) 2004-2008 Petr Lorenz
# All rights reserved.
# 
# ##### BEGIN LICENSE BLOCK #####
#
# Version: MPL 1.1 / GPL 2.0 / LGPL 2.1
#
# The contents of this file are subject to the Mozilla Public License
# Version 1.1 (the "License"); you may not use this file except in
# compliance with the License. You may obtain a copy of the License at
# http://www.mozilla.org/MPL/
# 
# Software distributed under the License is distributed on an "AS IS"
# basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
# License for the specific language governing rights and limitations
# under the License.
# 
# The Original Code is Petr Lorenz emGine Graphical User Interface (GUI).
# 
# The Initial Developer of the Original Code is Petr Lorenz.
# Portions created by the Initial Developer are Copyright (C) 2004-2008.
# All Rights Reserved.
# 
# Alternatively, the contents of this file may be used under the terms of
# either the GNU General Public License Version 2 or later (the  "GPL"), or the
# GNU Lesser General Public License Version 2.1 or later (the "LGPL"), in which
# case the provisions of the GPL or the LGPL are applicable instead of those
# above.  If you wish to allow use of your version of this file only under the
# terms of either the GPL or the LGPL and not to allow others to use your
# version of this file under the MPL, indicate your decision by deleting the
# provisions above and replace them with the notice and other provisions
# required by the GPL or LGPL.  If you do not delete the provisions above, a
# recipient may use your version of this file under the terms of any one of the
# MPL License, the GPL or the LGPL.
#
# ##### END LICENSE BLOCK #####
#
#######################################################################
# 
# $Log: ResultImpedance.py,v $
# Revision 1.2  2008-02-07 05:45:25  lorenz
# * Fixed 'Simulation Boundary View' check problem
#   when closing a project.
#
# * Fixed Bounding Box problems when moving/editing a geometry.
#
# Revision 1.1.1.1  2008-01-28 06:15:35  lorenz
# Import emGine GUI v. 0.7.1
#
# Revision 1.2  2007-07-02 06:37:18  lorenz
# Clean up of old code.
#
# Revision 1.1.1.1  2007-05-08 06:22:35  lorenz
# Initial Import
#
#######################################################################

# July 2006 - April 2007

from Result import Result
from ResultSParameters import ResultSParameters

from XYData import XYData
from XYDataSource import XYDataSource

import math
import numpy

class ResultImpedance(Result):
  def __init__(self, input_data_ = '', name_ = 'Impedance'):
    Result.__init__(self, name_)
    self.type = "NotSpecified"
    self.impedance = None
    self.fmin = 1e8
    self.fmax = 1e9
    self.freq_points = 201
    self.Z0 = 50

    if isinstance(input_data_, ResultSParameters):
      self.type = "FromSParameters"
      self.impedance = XYData()
      self.input_data = input_data_
    else:
      self.type = "FromFileTDSignal"
      self.impedance = XYData()
      self.file_source = input_data_
      self.v_index = 2
      self.i_index = 3

  def set_fmin_fmax(self, fmin_, fmax_):
    self.fmin = float(fmin_)
    self.fmax = float(fmax_)

  def update(self):
    print "Updating ResultImpedance"

    if self.type == "FromSParameters":
      print "Updating impedance from SParameters!"
      spar_data = self.input_data.get_data()
      self.impedance.set_xvalues(spar_data.get_xarray())
      s11 = spar_data.get_yarray()
      self.impedance.set_yvalues( self.Z0*(1 + s11)/(1 - s11) )
      self.names.append("z11")
    
    elif self.type == "FromFileTDSignal":
      print "fmin =", self.fmin
      print "fmax =", self.fmax
      print "freq_points =", self.freq_points
      df = float((self.fmax - self.fmin)/self.freq_points)
      print "df =", df
      
      voltage = XYDataSource()
      voltage.read_from_ascii_file(self.file_source, self.v_index, 1)

      current = XYDataSource()
      current.read_from_ascii_file(self.file_source, self.i_index, 1)

      dt = voltage.XYData.get_xarray()[0]
      print "dt =", dt
      N = round(1/(df*dt))
      N_fft = int(2**(math.ceil(math.log(N)/math.log(2))))
      df_new = 1/(N_fft*dt)
      print "df_new =", df_new
      print "N_fft =", N_fft
      Nmin = math.floor(self.fmin/df_new) 
      Nmax = math.ceil(self.fmax/df_new)
      self.freq_points = int(Nmax - Nmin + 1)
      print "CP1"
      # the frequency computation is a little bit tricky to avoid numerical problems
      
      # Note: In numpy 1.0.1 the name of the function arrayrange was changed to arange 
      #
      #self.impedance.set_xvalues(df_new*numpy.arrayrange(Nmin, Nmax+1))       
      self.impedance.set_xvalues(df_new*numpy.arange(Nmin, Nmax+1))       

      #voltage.print_out()
      #current.print_out()

      # Fourier transform the voltage and the current
      voltage.XYData.fft(N_fft)
      current.XYData.fft(N_fft)
      #voltage.XYData.fft()
      #current.XYData.fft()

      #voltage.print_out()
      #current.print_out()

      Z = -1*voltage.XYData.get_yarray()/(2*current.XYData.get_yarray()) 
      print "CP2"

      self.impedance.add_yvalues(Z[Nmin:Nmax+1])

      #return self.impedance

  def get_data(self):
    return self.impedance

###############################################
if __name__ == '__main__':
  print "Testing ResultImpedance Module..."

  res = ResultImpedance('../tmp/EH.Dipole1.emm')
  res.fmin = 3e8
  res.fmax = 1e9
  #res.update()



#######################################################################
#
# $Id: Project.py,v 1.4 2008-03-28 06:25:37 lorenz Exp $
# 
# Copyright (C) 2004-2008 Petr Lorenz
# All rights reserved.
# 
# ##### BEGIN LICENSE BLOCK #####
#
# Version: MPL 1.1 / GPL 2.0 / LGPL 2.1
#
# The contents of this file are subject to the Mozilla Public License
# Version 1.1 (the "License"); you may not use this file except in
# compliance with the License. You may obtain a copy of the License at
# http://www.mozilla.org/MPL/
# 
# Software distributed under the License is distributed on an "AS IS"
# basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
# License for the specific language governing rights and limitations
# under the License.
# 
# The Original Code is Petr Lorenz emGine Graphical User Interface (GUI).
# 
# The Initial Developer of the Original Code is Petr Lorenz.
# Portions created by the Initial Developer are Copyright (C) 2004-2008.
# All Rights Reserved.
# 
# Contributor(s): Bruno Biscontini, Rachid Khlifi.
# 
# Alternatively, the contents of this file may be used under the terms of
# either the GNU General Public License Version 2 or later (the  "GPL"), or the
# GNU Lesser General Public License Version 2.1 or later (the "LGPL"), in which
# case the provisions of the GPL or the LGPL are applicable instead of those
# above.  If you wish to allow use of your version of this file only under the
# terms of either the GPL or the LGPL and not to allow others to use your
# version of this file under the MPL, indicate your decision by deleting the
# provisions above and replace them with the notice and other provisions
# required by the GPL or LGPL.  If you do not delete the provisions above, a
# recipient may use your version of this file under the terms of any one of the
# MPL License, the GPL or the LGPL.
#
# ##### END LICENSE BLOCK #####
#
#######################################################################
# 
# $Log: Project.py,v $
# Revision 1.4  2008-03-28 06:25:37  lorenz
# - Perspective/Orthogonal view
# - Top/Bottom/Front/Back/Left/Right views
# - Boundary conditions setup
# - Platform Info
#
# Revision 1.3  2008-02-07 05:45:25  lorenz
# * Fixed 'Simulation Boundary View' check problem
#   when closing a project.
#
# * Fixed Bounding Box problems when moving/editing a geometry.
#
# Revision 1.2  2008-01-28 16:14:38  lorenz
# Main tag of EMM file changed to 'emGine'
#
# Revision 1.1.1.1  2008-01-28 06:15:35  lorenz
# Import emGine GUI v. 0.7.1
#
# Revision 1.6  2007-07-02 06:37:18  lorenz
# Clean up of old code.
#
# Revision 1.5  2007-06-18 15:55:56  lorenz
# Bugfix for reading a surrounding space with 'None' entries
#
# Revision 1.4  2007-06-14 06:31:51  lorenz
# Added emGine settings handling.
# Better handling of GUI state through GUIState class.
#
# Revision 1.3  2007-05-18 16:37:01  lorenz
# Work on 'surrounding space' simbox and
#  'emgine_gui' tag
#
# Revision 1.2  2007-05-14 04:53:51  lorenz
# New code for multiport time-domain signals and S-parameters
#
# Revision 1.1.1.1  2007-05-08 06:22:35  lorenz
# Initial Import
#
#######################################################################

# January 2006 - March 2008

import os
from xml.dom.minidom import parse, parseString, Node

from SurroundingSpace import SurroundingSpace

import consts

############################################
## Project class
############################################
class Project:
  def __init__(self):
    self.state = consts.PRJ_STATE_INITIALIZED
    self.fname = consts.PRJ_DEFAULT_FNAME
    self.dom = None

  def new(self, prj_args):
    if self.state is not consts.PRJ_STATE_INITIALIZED:
      return
    #new_project = "<emGine version=\"%s\"><project name=\"%s\" units=\"%s\">%s</project></emGine>" %(consts.PRJ_TLMGUI_VERSION, prj_args[0], prj_args[1], consts.DEFAULT_PROJECT)
    new_project = "<emGine version=\"%s\"><project name=\"%s\">%s</project></emGine>" %(consts.PRJ_TLMGUI_VERSION, prj_args[0], consts.DEFAULT_PROJECT)
    #new_project = "<?xml version=\"1.0\" encoding=\"ISO-8859-1\"?>\n<emGine/>"
    #print new_project
    self.dom = parseString(new_project)

    self.set_units(prj_args[1:])
    self.state = consts.PRJ_STATE_NEW
    print "Defining new project"

  ###------------------------------
  ### open_working_file
  ###------------------------------
  def open_working_file(self, fname_):
    if self.state is not consts.PRJ_STATE_INITIALIZED:
      return

    # change the working directory
    pth = os.path.split(fname_)
    print ">> New CWD: ", pth[0]
    os.chdir(pth[0])

    self.fname = pth[1]
    self.dom = parse(self.fname)
    #print self.dom.toxml()
    # TODO: check here for the version
    #print self.dom.getElementsByTagName("emGine")
    self.state = consts.PRJ_STATE_OPENED


  ###------------------------------
  ### save
  ###------------------------------
  def save(self, fname_ = None):
    if not fname_:
      fname_ = self.fname
    f = open(fname_, "w")
    self.dom.writexml(f, indent = "")
    f.close()

    self.fname = fname_
    self.state = consts.PRJ_STATE_SAVED

    # change the working directory
    pth = os.path.split(fname_)
    dir = pth[0]
    if dir:
      os.chdir(pth[0])

  #
  # return True, if the project has been modified
  #
  def modified(self):
    #print "Project::modified called! state = " + str(self.state)
    if self.state == consts.PRJ_STATE_MODIFIED:
      return True
    if self.state == consts.PRJ_STATE_NEW:
      return True
    return False

  ###---------------
  ### get_file_name
  ###---------------
  def get_file_name(self):
    pth = os.path.split(self.fname)
    return pth[1]

  ###--------------------------------
  ### get_project_name
  ###
  ### Return the name of the project
  ###--------------------------------
  def get_project_name(self):
    if not self.dom:
      return ''

    project_node = self.dom.getElementsByTagName(consts.XML_TAG_PROJECT)[0]    # !! TODO: only the first project !
    return project_node.getAttribute(consts.XML_ATTR_NAME)

  ###-----------
  ### get_units
  ###-----------
  def get_units(self):
    if not self.dom:
      return None

    # project_node = self.dom.getElementsByTagName(consts.XML_TAG_PROJECT)[0]    # !! TODO: only the first project !
    # return float(project_node.getAttribute(consts.XML_ATTR_UNITS))

    units_node = self.dom.getElementsByTagName(consts.XML_TAG_UNITS)
    units = {}
    for node in units_node.item(0).childNodes:
      if node.nodeType == Node.ELEMENT_NODE:
	value_node = node.childNodes[0]
	units[node.localName] = float(value_node.nodeValue)

    #print units
    return units


  ###------------------------
  ### get 2D point from node
  ###------------------------
  def get_point2d(self, node):
    point = None
    comp_v = {consts.XML_TAG_POINT2D_COMPONENTS[0] : None, consts.XML_TAG_POINT2D_COMPONENTS[1] : None}   # verbose components in a dictionary
    if node.localName == consts.XML_TAG_POINT2D:
      #print "Project::get_point2d - reading point"
      for data in node.childNodes:
	if data.nodeType == Node.ELEMENT_NODE:
	  if data.localName in consts.XML_TAG_POINT2D_COMPONENTS:
	    value_node = data.childNodes[0]
	    comp_v[data.localName] = value_node.nodeValue

    if None not in comp_v.values():
      point = [0.0, 0.0]
      for ind in range(2):
	point[ind] = float(comp_v[consts.XML_TAG_POINT2D_COMPONENTS[ind]])

    return point

  # get 3D point from node
  def get_point3d(self, node):
    point = None
    comp_v = {consts.XML_TAG_POINT3D_COMPONENTS[0] : None, consts.XML_TAG_POINT3D_COMPONENTS[1] : None, consts.XML_TAG_POINT3D_COMPONENTS[2] : None}   # verbose components in a dictionary
    if node.localName == consts.XML_TAG_POINT3D:
      #print "Project::get_point3d - reading point"
      for data in node.childNodes:
	if data.nodeType == Node.ELEMENT_NODE:
	  if data.localName in consts.XML_TAG_POINT3D_COMPONENTS:
	    value_node = data.childNodes[0]
	    comp_v[data.localName] = value_node.nodeValue

    if None not in comp_v.values():
      point = [0.0, 0.0, 0.0]
      for ind in range(3):
	point[ind] = float(comp_v[consts.XML_TAG_POINT3D_COMPONENTS[ind]])

    return point

  # read a list of 2D points
  def get_list_points2d(self, node):
    points = None
    if node.localName == consts.XML_TAG_POINTS:
      points = []
      for data in node.childNodes:
	if data.nodeType == Node.ELEMENT_NODE:
	  point = self.get_point2d(data)
	  if point:
	    points.append(point)

      if len(points) == 0:
	points = None
    return points

  # get 2D vector components from node
  def get_vector2d(self, node):
    vector = None
    comp_v = {consts.XML_TAG_VECTOR2D_COMPONENTS[0] : None, consts.XML_TAG_VECTOR2D_COMPONENTS[1] : None}   # verbose components in a dictionary
    if node.localName == consts.XML_TAG_VECTOR2D:
      #print "Project::get_vector2d - reading vector"
      for data in node.childNodes:
	if data.nodeType == Node.ELEMENT_NODE:
	  if data.localName in consts.XML_TAG_VECTOR2D_COMPONENTS:
	    value_node = data.childNodes[0]
	    comp_v[data.localName] = value_node.nodeValue

    if None not in comp_v.values():
      vector = [0.0, 0.0]
      for ind in range(2):
	vector[ind] = float(comp_v[consts.XML_TAG_VECTOR2D_COMPONENTS[ind]])

    return vector

  # get 3D vector components from node
  def get_vector3d(self, node):
    vector = None
    comp_v = {consts.XML_TAG_VECTOR3D_COMPONENTS[0] : None, consts.XML_TAG_VECTOR3D_COMPONENTS[1] : None, consts.XML_TAG_VECTOR3D_COMPONENTS[2] : None}   # verbose components in a dictionary
    if node.localName == consts.XML_TAG_VECTOR3D:
      #print "Project::get_vector3d - reading vector"
      for data in node.childNodes:
	if data.nodeType == Node.ELEMENT_NODE:
	  if data.localName in consts.XML_TAG_VECTOR3D_COMPONENTS:
	    value_node = data.childNodes[0]
	    comp_v[data.localName] = value_node.nodeValue

    if None not in comp_v.values():
      vector = [0.0, 0.0, 0.0]
      for ind in range(3):
	vector[ind] = float(comp_v[consts.XML_TAG_VECTOR3D_COMPONENTS[ind]])

    return vector

  def get_all_geometries(self):
    # FIXME: this routine is q&d

    #print "Project::get_all_geometries returning a list of all geometrical objects"
    geometry = self.dom.getElementsByTagName(consts.XML_TAG_GEOMETRY)
    geom_list = list()
    for node in geometry.item(0).childNodes:
      if node.nodeType == Node.ELEMENT_NODE:
        object = {}
        object[consts.XML_ATTR_NAME] = node.getAttribute(consts.XML_ATTR_NAME)
	object[consts.XML_ATTR_GEOMETRY_MATERIAL] = node.getAttribute(consts.XML_ATTR_GEOMETRY_MATERIAL)

        object[consts.XML_TAG_GEOMETRY] = node.localName
	if node.localName == consts.XML_TAG_POLYGON2D:
	  print "Project::get_all_geometries reading polygon"
	  for data in node.childNodes:
	    if data.nodeType == Node.ELEMENT_NODE:
	      if data.localName == consts.XML_TAG_NORMAL:
		#print "Reading normal"
		normal = None
		for ndata in data.childNodes:
		  if ndata.nodeType == Node.ELEMENT_NODE:
		    normal = self.get_vector3d(ndata)
		    break
		if not normal:
		  print "!! Warning !! Definition of normal for POLYGON2D not found!"
		else:
		  object[data.localName] = normal
	      elif data.localName == consts.XML_TAG_LEVEL:
		#print "Reading level"
		value_node = data.childNodes[0]
		level = float(value_node.nodeValue)
		object[data.localName] = level
	      elif data.localName == consts.XML_TAG_POINTS:
		#print "Reading points"
		points = self.get_list_points2d(data)
		if not points:
		  print "!! Warning !! Could not read point list for POLYGON2D!"
		else:
		  object[data.localName] = points
	      else:
		print "!! Warning !! >>", data.localName, "<< Unknown definition for POLYGON2D!"
	else:
	  for data in node.childNodes:
	    if data.nodeType == Node.ELEMENT_NODE:
	      if data.hasChildNodes():
		#
		# FIXME: If ELEMENT_NODE type is a Child
		#        then the value is not necessarily the
		#        first child!, i.e., childNodes[0]
		#
		value = ""
		value_node = data.childNodes[0]
		if value_node.nodeType == Node.TEXT_NODE:
		  value = value_node.nodeValue
		elif value_node.nodeType == Node.ELEMENT_NODE:
		  value = value_node.localName
		object[data.localName] = value
	object[consts.GUI_XML_NODE] = node
        geom_list.append(object)

    return geom_list 

  def get_all_materials(self):
    # FIXME: this routine is q&d
    #
    #print "Project::get_all_materials returning a list of all materials objects"
    material = self.dom.getElementsByTagName(consts.XML_TAG_MATERIALS)
    material_list = list()
    for node in material.item(0).childNodes:
      if node.nodeType == Node.ELEMENT_NODE:
        object = {}
        object[consts.XML_ATTR_NAME] = node.getAttribute(consts.XML_ATTR_NAME)
        object[consts.XML_TAG_MATERIALS] = node.localName
        for data in node.childNodes:
          if data.nodeType == Node.ELEMENT_NODE:
            if data.hasChildNodes():
              #
              # FIXME: If ELEMENT_NODE type is a Child
              #        then the value is not necessarily the
              #        first child!, i.e., childNodes[0]
              #
              value = ""
              value_node = data.childNodes[0]
              if value_node.nodeType == Node.TEXT_NODE:
                value = value_node.nodeValue
              elif value_node.nodeType == Node.ELEMENT_NODE:
                value = value_node.localName
              object[data.localName] = value
	object[consts.GUI_XML_NODE] = node
        material_list.append(object)

    return material_list 

  def get_all_excitations(self):
    print "Project::get_all_excitations returning a list of all excitation objects"
    excitation = self.dom.getElementsByTagName(consts.XML_TAG_EXCITATION)
    excit_list = list()		# the list will be returned
    for node in excitation.item(0).childNodes:
      if node.nodeType == Node.ELEMENT_NODE:
	object = {}		# associative list
	object[consts.XML_ATTR_NAME] = node.getAttribute(consts.XML_ATTR_NAME)
	object[consts.XML_TAG_EXCITATION] = node.localName
	for data in node.childNodes:
	  if data.nodeType == Node.ELEMENT_NODE:
	    if data.hasChildNodes():
              #
              # FIXME: If ELEMENT_NODE type is a Child
              #        then the value is not necessarily the
              #        first child!, i.e., childNodes[0]
              #
	      value = ""
	      value_node = data.childNodes[0]
	      if value_node.nodeType == Node.TEXT_NODE:
		value = value_node.nodeValue
	      elif value_node.nodeType == Node.ELEMENT_NODE:
		value = value_node.localName
	      object[data.localName] = value
	object[consts.GUI_XML_NODE] = node
	excit_list.append(object)

    return excit_list

  def get_all_results(self, type):
    results = self.dom.getElementsByTagName(consts.XML_TAG_RESULTS)
    results_list = list()		# the list will be returned
    if results:
      for node in results.item(0).childNodes:
	if node.nodeType == Node.ELEMENT_NODE:
	  if node.getAttribute(consts.XML_ATTR_RESULT_TYPE) == type:		# is this the result of the given type?
	    for resnode in node.childNodes:
	      if resnode.nodeType == Node.ELEMENT_NODE:
		object = {}		# associative list
		object[consts.XML_ATTR_NAME] = resnode.getAttribute(consts.XML_ATTR_NAME)
		object[consts.XML_TAG_RESULTS] = resnode.localName
		for data in resnode.childNodes:
		  if data.nodeType == Node.ELEMENT_NODE:
		    if data.hasChildNodes():
		      #
		      # FIXME: If ELEMENT_NODE type is a Child
		      #        then the value is not necessarily the
		      #        first child!, i.e., childNodes[0]
		      #
		      value = ""
		      value_node = data.childNodes[0]
		      if value_node.nodeType == Node.TEXT_NODE:
			value = value_node.nodeValue
		      elif value_node.nodeType == Node.ELEMENT_NODE:
			value = value_node.localName
		      object[data.localName] = value
		object[consts.GUI_XML_NODE] = resnode
		results_list.append(object)

    return results_list

  ### Return the settings of the emGine GUI
  def get_emginegui_settings(self):
    emginegui_settings = self.dom.getElementsByTagName(consts.XML_TAG_EMGINE_GUI)
    settings = 	{}	# store all the settings in this dictionary
    if len(emginegui_settings) != 0:	# only if not empty
      for node in emginegui_settings.item(0).childNodes:
	if node.nodeType == Node.ELEMENT_NODE:
	  setting = {}	# dictionary for a setting
	  if node.localName == consts.XML_TAG_SURROUNDING_SPACE:
	    print "Reading surrounding space settings" 
	    setting = self.get_data_as_dictionary(node)

	  settings[node.localName] = setting

    return settings

  # get all file sources defined as children of the node
  def get_file_sources(self, node):
    files = list()
    for child_node in node.childNodes:
      if child_node.nodeType == Node.ELEMENT_NODE:
	if child_node.localName == consts.XML_TAG_FILE:
	  files.append(child_node.getAttribute(consts.XML_ATTR_FILE_SRC))
    return files

  # return a dictionary of results based on the 
  # information in the DOM
  def get_results(self, type = consts.RESULT_TYPE_SIMULATION):
    results = {}
    results["tds_file"] = list()	# erase the list of files
    #results["resobjs"] = list()		# just for testing !!!
    #results["tdsignals"] = list()	# list of time-domain signals
    #results["sparams"] = list()		# list of S-parameters
    #results["zparams"] = list()	# list of impedances

    results_list = self.get_all_results(type)
    #print results_list
    for result in results_list:
      node = result[consts.GUI_XML_NODE]
      if node.localName == consts.XML_TAG_TDSIGNAL:
	for fsource in self.get_file_sources(node):
	  results["tds_file"].append(fsource)

    if len(results["tds_file"]) == 0:	# return None if no results available
      return None
    return results

  # return a dictionary of reference results
  # based on the information in the DOM
  #
  # if no reference - return None
  def get_ref_results(self):
    return self.get_results(consts.RESULT_TYPE_REFERENCE)

  def get_surrounding_space(self):
    # FIXME: this routine not done
    #print "Project::get_surrounding_space called"
    settings = self.get_emginegui_settings()
    if len(settings):
      surr_space = settings[consts.XML_TAG_SURROUNDING_SPACE]

      print surr_space
      box = [None, None, None, None, None, None]
      box[0] = surr_space[consts.XML_TAG_X1]
      box[1] = surr_space[consts.XML_TAG_Y1]
      box[2] = surr_space[consts.XML_TAG_Z1]
      box[3] = surr_space[consts.XML_TAG_X2]
      box[4] = surr_space[consts.XML_TAG_Y2]
      box[5] = surr_space[consts.XML_TAG_Z2]

      for i in range(len(box)):
	box_e = box[i]
	if (not box_e) or (box_e == 'None'):
	  box[i] = None
	else:
	  box[i] = float(box_e)
	       
      surrounding_space = SurroundingSpace([[box[0], box[1], box[2]], [box[3], box[4], box[5]]])
    else:
      surrounding_space = SurroundingSpace()

    return surrounding_space
             
  def delete_geometry_node(self, node):
    self.state = consts.PRJ_STATE_MODIFIED

    geometry = self.dom.getElementsByTagName(consts.XML_TAG_GEOMETRY)
    geometry[0].removeChild(node)       # we use only one geometry block in EMM file
    # FIXME: unlink method should be called?!

  def delete_material_node(self, node):
    self.state = consts.PRJ_STATE_MODIFIED

    materials = self.dom.getElementsByTagName(consts.XML_TAG_MATERIALS)
    materials[0].removeChild(node)       # we use only one material block in EMM file
    # FIXME: unlink method should be called?!

  def delete_excitation_node(self, node):
    self.state = consts.PRJ_STATE_MODIFIED

    excitations = self.dom.getElementsByTagName(consts.XML_TAG_EXCITATION)
    excitations[0].removeChild(node)       # we use only one material block in EMM file
    # FIXME: unlink method should be called?!

  ### FIXME: should the method 'generate_patch_element' 
  ###        be a method of GeometryObjectPatch ?
  def generate_patch_element(self, data):
    name = data[0]
    x1 = data[1]
    y1 = data[2]
    z1 = data[5]
    length_x= data[3]
    length_y = data[4]
    material = data[-1]

    new_geometry = self.dom.createElement(consts.XML_TAG_PATCH)
    attr_geometry = self.dom.createAttribute(consts.XML_ATTR_NAME)
    new_geometry.setAttributeNode(attr_geometry)
    new_geometry.setAttribute(consts.XML_ATTR_NAME, name)

    self.set_attr(new_geometry, consts.XML_ATTR_GEOMETRY_MATERIAL, material)

    x1_node = self.dom.createElement(consts.XML_TAG_X1)
    x1_text = self.dom.createTextNode(x1)
    x1_node.appendChild(x1_text)

    y1_node = self.dom.createElement(consts.XML_TAG_Y1)
    y1_text = self.dom.createTextNode(y1)
    y1_node.appendChild(y1_text)

    z1_node = self.dom.createElement(consts.XML_TAG_Z1)
    z1_text = self.dom.createTextNode(z1)
    z1_node.appendChild(z1_text)

    length_x_node = self.dom.createElement(consts.XML_TAG_LENGTH_X)
    length_x_text = self.dom.createTextNode(length_x)
    length_x_node.appendChild(length_x_text)

    length_y_node = self.dom.createElement(consts.XML_TAG_LENGTH_Y)
    length_y_text = self.dom.createTextNode(length_y)
    length_y_node.appendChild(length_y_text)

    new_geometry.appendChild(x1_node)
    new_geometry.appendChild(y1_node)
    new_geometry.appendChild(z1_node)
    new_geometry.appendChild(length_x_node)
    new_geometry.appendChild(length_y_node)

    return new_geometry

  def add_geometry_patch(self, data):
    self.state = consts.PRJ_STATE_MODIFIED
    
    # update DOM
    geometry_node = self.dom.getElementsByTagName(consts.XML_TAG_GEOMETRY)[0]
    new_geometry = self.generate_patch_element(data)
    geometry_node.appendChild(new_geometry)

    #print geometry_node

  def generate_circle_element(self, data):
    name = data[0]
    x1 = data[1]
    y1 = data[2]
    radius = data[3]
    z1 = data[4]
    material = data[-1]

    new_geometry = self.dom.createElement(consts.XML_TAG_CIRCLE)
    attr_geometry = self.dom.createAttribute(consts.XML_ATTR_NAME)
    new_geometry.setAttributeNode(attr_geometry)
    new_geometry.setAttribute(consts.XML_ATTR_NAME, name)

    self.set_attr(new_geometry, consts.XML_ATTR_GEOMETRY_MATERIAL, material)

    x1_node = self.dom.createElement(consts.XML_TAG_X1)
    x1_text = self.dom.createTextNode(x1)
    x1_node.appendChild(x1_text)

    y1_node = self.dom.createElement(consts.XML_TAG_Y1)
    y1_text = self.dom.createTextNode(y1)
    y1_node.appendChild(y1_text)

    z1_node = self.dom.createElement(consts.XML_TAG_Z1)
    z1_text = self.dom.createTextNode(z1)
    z1_node.appendChild(z1_text)

    orientation_node = self.dom.createElement(consts.XML_TAG_CYLINDER_ORIENTATION)
    orientation_text = self.dom.createElement(consts.XML_TAG_ORIENTATION_Z)
    orientation_node.appendChild(orientation_text)
    
    radius_node = self.dom.createElement(consts.XML_TAG_RADIUS)
    radius_text = self.dom.createTextNode(radius)
    radius_node.appendChild(radius_text)

    new_geometry.appendChild(x1_node)
    new_geometry.appendChild(y1_node)
    new_geometry.appendChild(z1_node)
    new_geometry.appendChild(orientation_node)
    new_geometry.appendChild(radius_node)

    return new_geometry

  def add_geometry_circle(self, data):
    self.state = consts.PRJ_STATE_MODIFIED

    # update DOM
    geometry_node = self.dom.getElementsByTagName(consts.XML_TAG_GEOMETRY)[0]
    new_geometry = self.generate_circle_element(data)
    geometry_node.appendChild(new_geometry)

  def generate_polygon_element(self, data):
    points = data[0]
    data_text = data[1]
    normal_v = data[2]

    name = data_text[0]
    level = float(data_text[1])

    material = data[-1]

    new_geometry = self.dom.createElement(consts.XML_TAG_POLYGON2D)
    attr_geometry = self.dom.createAttribute(consts.XML_ATTR_NAME)
    new_geometry.setAttributeNode(attr_geometry)
    new_geometry.setAttribute(consts.XML_ATTR_NAME, name)

    self.set_attr(new_geometry, consts.XML_ATTR_GEOMETRY_MATERIAL, material)

    # normal
    normal_node = self.dom.createElement(consts.XML_TAG_NORMAL)
    normal = [0.0, 0.0, 1.0]
    if normal_v == 'x':
      normal = [1.0, 0.0, 0.0]
    elif normal_v == 'y':
      normal = [0.0, 1.0, 0.0]
    vect_node = self.dom.createElement(consts.XML_TAG_VECTOR3D)
    for ind in range(3):
      comp_node = self.dom.createElement(consts.XML_TAG_VECTOR3D_COMPONENTS[ind])
      comp_text = self.dom.createTextNode(str(normal[ind]))
      comp_node.appendChild(comp_text)
      vect_node.appendChild(comp_node)
    normal_node.appendChild(vect_node)

    # level
    level_node = self.dom.createElement(consts.XML_TAG_LEVEL)
    level_text = self.dom.createTextNode(str(level))
    level_node.appendChild(level_text)

    # points
    points_node = self.dom.createElement(consts.XML_TAG_POINTS)
    for point in points:
      point_node = self.dom.createElement(consts.XML_TAG_POINT2D)
      for ind in range(2):
	comp_node = self.dom.createElement(consts.XML_TAG_VECTOR3D_COMPONENTS[ind])
	comp_text = self.dom.createTextNode(str(point[ind]))
	comp_node.appendChild(comp_text)
	point_node.appendChild(comp_node)
      points_node.appendChild(point_node)

    # append new tags
    new_geometry.appendChild(normal_node)
    new_geometry.appendChild(level_node)
    new_geometry.appendChild(points_node)

    return new_geometry

  def add_geometry_polygon(self, data):
    self.state = consts.PRJ_STATE_MODIFIED

    #print "Project::add_geometry_polygon - ", name, normal_v, level, points

    # update DOM
    geometry_node = self.dom.getElementsByTagName(consts.XML_TAG_GEOMETRY)[0]
    new_geometry = self.generate_polygon_element(data)
    geometry_node.appendChild(new_geometry)

  def generate_box_element(self, data):
    name = data[0]
    x1 = data[1]
    y1 = data[2]
    z1 = data[3]
    length_x= data[4]
    length_y = data[5]
    length_z = data[6]
    material = data[-1]

    new_geometry = self.dom.createElement(consts.XML_TAG_CUBE)
    attr_geometry = self.dom.createAttribute(consts.XML_ATTR_NAME)
    new_geometry.setAttributeNode(attr_geometry)
    new_geometry.setAttribute(consts.XML_ATTR_NAME, name)

    self.set_attr(new_geometry, consts.XML_ATTR_GEOMETRY_MATERIAL, material)

    x1_node = self.dom.createElement(consts.XML_TAG_X1)
    x1_text = self.dom.createTextNode(x1)
    x1_node.appendChild(x1_text)

    y1_node = self.dom.createElement(consts.XML_TAG_Y1)
    y1_text = self.dom.createTextNode(y1)
    y1_node.appendChild(y1_text)

    z1_node = self.dom.createElement(consts.XML_TAG_Z1)
    z1_text = self.dom.createTextNode(z1)
    z1_node.appendChild(z1_text)

    length_x_node = self.dom.createElement(consts.XML_TAG_LENGTH_X)
    length_x_text = self.dom.createTextNode(length_x)
    length_x_node.appendChild(length_x_text)

    length_y_node = self.dom.createElement(consts.XML_TAG_LENGTH_Y)
    length_y_text = self.dom.createTextNode(length_y)
    length_y_node.appendChild(length_y_text)

    length_z_node = self.dom.createElement(consts.XML_TAG_LENGTH_Z)
    length_z_text = self.dom.createTextNode(length_z)
    length_z_node.appendChild(length_z_text)

    new_geometry.appendChild(x1_node)
    new_geometry.appendChild(y1_node)
    new_geometry.appendChild(z1_node)
    new_geometry.appendChild(length_x_node)
    new_geometry.appendChild(length_y_node)
    new_geometry.appendChild(length_z_node)

    return new_geometry

  def add_geometry_box(self, data):
    self.state = consts.PRJ_STATE_MODIFIED

    # update DOM
    geometry_node = self.dom.getElementsByTagName(consts.XML_TAG_GEOMETRY)[0]
    new_geometry = self.generate_box_element(data)
    geometry_node.appendChild(new_geometry)

  def generate_cylinder_element(self, data):
    name = data[0]
    x1 = data[1]
    y1 = data[2]
    z1 = data[3]
    radius = data[4]
    height = data[5]
    material = data[-1]

    new_geometry = self.dom.createElement(consts.XML_TAG_CYLINDER)
    attr_geometry = self.dom.createAttribute(consts.XML_ATTR_NAME)
    new_geometry.setAttributeNode(attr_geometry)
    new_geometry.setAttribute(consts.XML_ATTR_NAME, name)

    self.set_attr(new_geometry, consts.XML_ATTR_GEOMETRY_MATERIAL, material)

    x1_node = self.dom.createElement(consts.XML_TAG_X1)
    x1_text = self.dom.createTextNode(x1)
    x1_node.appendChild(x1_text)

    y1_node = self.dom.createElement(consts.XML_TAG_Y1)
    y1_text = self.dom.createTextNode(y1)
    y1_node.appendChild(y1_text)

    z1_node = self.dom.createElement(consts.XML_TAG_Z1)
    z1_text = self.dom.createTextNode(z1)
    z1_node.appendChild(z1_text)

    orientation_node = self.dom.createElement(consts.XML_TAG_CYLINDER_ORIENTATION)
    orientation_text = self.dom.createElement(consts.XML_TAG_ORIENTATION_Z)
    orientation_node.appendChild(orientation_text)
    
    radius_node = self.dom.createElement(consts.XML_TAG_RADIUS)
    radius_text = self.dom.createTextNode(radius)
    radius_node.appendChild(radius_text)

    height_node = self.dom.createElement(consts.XML_TAG_HEIGHT)
    height_text = self.dom.createTextNode(height)
    height_node.appendChild(height_text)

    new_geometry.appendChild(x1_node)
    new_geometry.appendChild(y1_node)
    new_geometry.appendChild(z1_node)
    new_geometry.appendChild(orientation_node)
    new_geometry.appendChild(radius_node)
    new_geometry.appendChild(height_node)

    return new_geometry

  def add_geometry_cylinder(self, data):
    self.state = consts.PRJ_STATE_MODIFIED
    
    # update DOM
    geometry_node = self.dom.getElementsByTagName(consts.XML_TAG_GEOMETRY)[0]
    new_geometry = self.generate_cylinder_element(data)
    geometry_node.appendChild(new_geometry)

  def replace_geometry_general(self, data, old_name, geometry_tag, geometry_generating_function):
    self.state = consts.PRJ_STATE_MODIFIED

    geometry_node = self.dom.getElementsByTagName(consts.XML_TAG_GEOMETRY)[0]
    old_node = self.get_node_with_attr(geometry_tag, consts.XML_ATTR_NAME, old_name, geometry_node)
    new_geometry = geometry_generating_function(data)
    geometry_node.replaceChild(new_geometry, old_node)

  def replace_geometry_patch(self, data, old_name):
    self.replace_geometry_general(data, old_name, consts.XML_TAG_PATCH, self.generate_patch_element)

  def replace_geometry_circle(self, data, old_name):
    self.replace_geometry_general(data, old_name, consts.XML_TAG_CIRCLE, self.generate_circle_element)

  def replace_geometry_polygon(self, data, old_name):
    self.replace_geometry_general(data, old_name, consts.XML_TAG_POLYGON2D, self.generate_polygon_element)

  def replace_geometry_box(self, data, old_name):
    self.replace_geometry_general(data, old_name, consts.XML_TAG_CUBE, self.generate_box_element)

  def replace_geometry_cylinder(self, data, old_name):
    self.replace_geometry_general(data, old_name, consts.XML_TAG_CYLINDER, self.generate_cylinder_element)

  #--
  def add_excitation_voltage_port(self, node, data):
    self.state = consts.PRJ_STATE_MODIFIED

    name = data[0]
    x1 = data[1]
    y1 = data[2]
    z1 = data[3]
    x2 = data[4]
    y2 = data[5]
    z2 = data[6]

    excitation_node = self.dom.getElementsByTagName(consts.XML_TAG_EXCITATION)[0]
    #print excitation_node

    new_excitation = self.dom.createElement(consts.XML_TAG_VOLTAGE_PORT)
    attr_geometry = self.dom.createAttribute(consts.XML_ATTR_NAME)
    new_excitation.setAttributeNode(attr_geometry)
    new_excitation.setAttribute(consts.XML_ATTR_NAME, name)

    x1_node = self.dom.createElement(consts.XML_TAG_X1)
    x1_text = self.dom.createTextNode(x1)
    x1_node.appendChild(x1_text)

    y1_node = self.dom.createElement(consts.XML_TAG_Y1)
    y1_text = self.dom.createTextNode(y1)
    y1_node.appendChild(y1_text)

    z1_node = self.dom.createElement(consts.XML_TAG_Z1)
    z1_text = self.dom.createTextNode(z1)
    z1_node.appendChild(z1_text)

    x2_node = self.dom.createElement(consts.XML_TAG_X2)
    x2_text = self.dom.createTextNode(x2)
    x2_node.appendChild(x2_text)

    y2_node = self.dom.createElement(consts.XML_TAG_Y2)
    y2_text = self.dom.createTextNode(y2)
    y2_node.appendChild(y2_text)

    z2_node = self.dom.createElement(consts.XML_TAG_Z2)
    z2_text = self.dom.createTextNode(z2)
    z2_node.appendChild(z2_text)

    new_excitation.appendChild(x1_node)
    new_excitation.appendChild(y1_node)
    new_excitation.appendChild(z1_node)
    new_excitation.appendChild(x2_node)
    new_excitation.appendChild(y2_node)
    new_excitation.appendChild(z2_node)

    excitation_node.appendChild(new_excitation)

  def set_boundary_conditions(self, data):
    self.state = consts.PRJ_STATE_MODIFIED

    boundaries_node = self.get_node(consts.XML_TAG_BOUNDARIES)

    data_ind = 0
    for boundary_name in [consts.XML_TAG_BOUNDARY_XMIN, consts.XML_TAG_BOUNDARY_XMAX, consts.XML_TAG_BOUNDARY_YMIN, consts.XML_TAG_BOUNDARY_YMAX, consts.XML_TAG_BOUNDARY_ZMIN, consts.XML_TAG_BOUNDARY_ZMAX]: 
      boundary_node = self.get_node_with_attr(consts.XML_TAG_BOUNDARY, consts.XML_ATTR_NAME, boundary_name, boundaries_node)
      bc_node = self.get_node(consts.XML_TAG_BC, boundary_node)
      self.set_attr(bc_node, consts.XML_BC_ATTR_TYPE, data[data_ind])
      #bc_node = self.get_node_with_attr(consts.XML_TAG_BC, consts.XML_BC_ATTR_TYPE, data[data_ind], boundary_node)
      data_ind += 1

  def get_boundary_conditions(self):
    # TODO: Check names of tags !!!!
    #       Better handling of incomplete BCs
    boundary_list = [consts.XML_TAG_BOUNDARY_XMIN, consts.XML_TAG_BOUNDARY_XMAX, consts.XML_TAG_BOUNDARY_YMIN, consts.XML_TAG_BOUNDARY_YMAX, consts.XML_TAG_BOUNDARY_ZMIN, consts.XML_TAG_BOUNDARY_ZMAX] 

    boundaries_node = self.get_node(consts.XML_TAG_BOUNDARIES)
    bc_dict = {}
    for node in boundaries_node.childNodes:
      if node.nodeType == Node.ELEMENT_NODE:
	if node.localName == consts.XML_TAG_BOUNDARY:
	  boundary_name = node.getAttribute(consts.XML_ATTR_NAME)
	  for data in node.childNodes:
	    if data.nodeType == Node.ELEMENT_NODE:
	      if data.localName == consts.XML_TAG_BC:
		bc_dict[boundary_name] = data.getAttribute(consts.XML_BC_ATTR_TYPE)
    bc_list = list()
    for bnd in boundary_list:
      try:
	bc_list.append(bc_dict[bnd])
      except:
	return None

    return bc_list

  def add_boundary(self, node, data):
    None
    #### self.state = consts.PRJ_STATE_MODIFIED

    #### name = data[0]
    #### typ = data[1] 
    #### ref  = data[2]

    #### boundary_node = self.dom.getElementsByTagName(consts.XML_TAG_BOUNDARIES)[0]
    #### #print boundary_node
    #### 
    #### new_boundary = self.dom.createElement(consts.XML_TAG_BOUNDARY)
    #### attr_geometry = self.dom.createAttribute(consts.XML_ATTR_NAME)
    #### new_boundary.setAttributeNode(attr_geometry)
    #### new_boundary.setAttribute(consts.XML_ATTR_NAME, name)

    #### typ_node = self.dom.createElement(consts.XML_TAG_TYP)
    #### typ_text = self.dom.createTextNode(typ)
    #### typ_node.appendChild(typ_text)
    #### 
    #### ref_node = self.dom.createElement(consts.XML_TAG_REF)
    #### ref_text = self.dom.createTextNode(ref)
    #### ref_node.appendChild(ref_text)
    #### 
    #### new_boundary.appendChild(typ_node)
    #### new_boundary.appendChild(ref_node)

    #### boundary_node.appendChild(new_boundary)

  def add_material(self, data):
    self.state = consts.PRJ_STATE_MODIFIED

    name = data[0] 
    epsr = data[1] 
    mur  = data[2] 
    cond = data[3] 
    tand = data[4] 
    
    material_node = self.dom.getElementsByTagName(consts.XML_TAG_MATERIALS)[0]
    #print material_node

    new_material = self.dom.createElement(consts.XML_TAG_MATERIAL)
    attr_geometry = self.dom.createAttribute(consts.XML_ATTR_NAME)
    new_material.setAttributeNode(attr_geometry)
    new_material.setAttribute(consts.XML_ATTR_NAME, name)
    
    epsr_node = self.dom.createElement(consts.XML_TAG_EPSR)
    epsr_text = self.dom.createTextNode(epsr)
    epsr_node.appendChild(epsr_text)
    
    mur_node = self.dom.createElement(consts.XML_TAG_MUR)
    mur_text = self.dom.createTextNode(mur)
    mur_node.appendChild(mur_text)

    cond_node = self.dom.createElement(consts.XML_TAG_CONDUCTIVITY)
    cond_text = self.dom.createTextNode(cond)
    cond_node.appendChild(cond_text)

    tand_node = self.dom.createElement(consts.XML_TAG_TAND)
    tand_text = self.dom.createTextNode(tand)
    tand_node.appendChild(tand_text)

    new_material.appendChild(epsr_node)
    new_material.appendChild(mur_node)
    new_material.appendChild(cond_node)
    new_material.appendChild(tand_node)

    material_node.appendChild(new_material)

  def set_units(self, new_units):
    length_u = new_units[0]
    freq_u = new_units[1]
    time_u = new_units[2]

    units_node = self.get_node(consts.XML_TAG_UNITS)
    self.set_value(units_node, consts.XML_TAG_UNITS_LENGTH, length_u)
    self.set_value(units_node, consts.XML_TAG_UNITS_FREQUENCY, freq_u)
    self.set_value(units_node, consts.XML_TAG_UNITS_TIME, time_u)

  def set_emgine_surrounding_space(self, simbox):
    """ Save the surrounding space (simbox) of the project """
    emgine_node = self.get_node(consts.XML_TAG_EMGINE_GUI)
    surrounding_space_node = self.get_node(consts.XML_TAG_SURROUNDING_SPACE, emgine_node)
    self.set_value(surrounding_space_node, consts.XML_TAG_X1, simbox[0][0])
    self.set_value(surrounding_space_node, consts.XML_TAG_Y1, simbox[0][1])
    self.set_value(surrounding_space_node, consts.XML_TAG_Z1, simbox[0][2])
    self.set_value(surrounding_space_node, consts.XML_TAG_X2, simbox[1][0])
    self.set_value(surrounding_space_node, consts.XML_TAG_Y2, simbox[1][1])
    self.set_value(surrounding_space_node, consts.XML_TAG_Z2, simbox[1][2])
    

  def set_signal_fmin_fmax(self, name, fmin, fmax):
    print "Setting fmin fmax for signal: ", name, " ", fmin, " ", fmax
    signals_node = self.get_node(consts.XML_TAG_SIGNALS)
    gaussmod_node = self.get_node(consts.XML_TAG_SIGNAL_GAUSSIAN_MODULATED, signals_node)
    self.set_value(gaussmod_node, consts.XML_TAG_FREQUENCY_MIN, fmin)
    self.set_value(gaussmod_node, consts.XML_TAG_FREQUENCY_MAX, fmax)

  def set_solver_number_of_time_steps(self, name, timesteps):
    print "Setting number of time steps", timesteps
    simulation_node = self.get_node(consts.XML_TAG_SIMULATION)
    tlm_node = self.get_node(consts.XML_TAG_SIMULATION_TLM, simulation_node)
    self.set_value(tlm_node, consts.XML_TAG_NUMBER_OF_TIME_STEPS, timesteps)

  # Description:
  # get the node with tag node_tag; if parent_node not specified,
  # take the dom root as the parent.
  # If the node does not exist, create a one.
  def get_node(self, node_tag, parent_node = None):
    if not parent_node:
      parent_node = self.dom.getElementsByTagName(consts.XML_TAG_PROJECT)[0]    # !! TODO: only the first project !

    nodes = parent_node.getElementsByTagName(node_tag)
    if (len(nodes) == 0):
      node = self.dom.createElement(node_tag)
      parent_node.appendChild(node)
    else:
      node = nodes[0]   # !! TODO: only the first one taken!!!

    return node

  # Description:
  # get the node with tag node_tag and node_attr with attr_value
  # if parent_node not specified,
  # take the dom root as the parent.
  # If the node does not exist, create a one.
  def get_node_with_attr(self, node_tag, node_attr, attr_value, parent_node = None):
    if not parent_node:
      parent_node = self.dom.getElementsByTagName(consts.XML_TAG_PROJECT)[0]    # !! TODO: only the first project !

    nodes = parent_node.getElementsByTagName(node_tag)

    for cn in nodes:
      if cn.getAttribute(node_attr) == attr_value:
	return cn	# node found, return it

    # node does not exist, create a one
    node = self.dom.createElement(node_tag)
    child_attr = self.dom.createAttribute(node_attr)
    node.setAttributeNode(child_attr)
    node.setAttribute(node_attr, attr_value)
    parent_node.appendChild(node)

    return node

  # Description:
  # set the value of the child node specified by child_tag 
  # of the parent node parent_node to value text node.
  # If the text node does not exist, or the child node does not
  # exist, they are created
  #
  # Return the child node
  def set_value(self, parent_node, child_tag, value):
    value = str(value)                                  # convert value to string
    value_tn = self.dom.createTextNode(value)
  
    child_nodes = parent_node.getElementsByTagName(child_tag)
    child_node = None
    if (len(child_nodes) == 0):
      child_node = self.dom.createElement(child_tag)
      parent_node.appendChild(child_node)
    else:
      child_node = child_nodes[0]       # !! TODO: taking only the first element, what if there are two or more?!

    if child_node.hasChildNodes():      # is there a value already set?
      old_value_tn = child_node.childNodes[0]
      child_node.replaceChild(value_tn, old_value_tn)
      old_value_tn.unlink()             # free from memory
    else:
      child_node.appendChild(value_tn)

    return child_node

  # Description:
  # Set the value of an attribute of a node
  def set_attr(self, node, attr_name_, attr_value_):
    attr_value = str(attr_value_)
    attr_name = str(attr_name_)

    if not node.getAttributeNode(attr_name):	# attribute does not exists ?
      child_attr = self.dom.createAttribute(attr_name)
      node.setAttributeNode(child_attr)

    node.setAttribute(attr_name, attr_value)


  # Description:
  # set the value of the child node specified by child_tag 
  # of the parent node parent_node to value text node.
  # If the text node does not exist, or the child node does not
  # exist, they are created
  # child_attr = (attr_name, attr_value)
  def set_value_for_attr(self, parent_node, child_tag, child_attr, value):
    value = str(value)                                  # convert value to string
    value_tn = self.dom.createTextNode(value)

    (attr_name, attr_value) = child_attr
    attr_name = str(attr_name)
    attr_value = str(attr_value)
  
    child_nodes = parent_node.getElementsByTagName(child_tag)
    child_node = None
    must_create_node = True
    if (len(child_nodes) != 0):
      for cn in child_nodes:
        if cn.getAttribute(attr_name) == attr_value:
          must_create_node = False
          child_node = cn
          break

    if must_create_node:
      child_node = self.dom.createElement(child_tag)
      child_attr = self.dom.createAttribute(attr_name)
      child_node.setAttributeNode(child_attr)
      child_node.setAttribute(attr_name, attr_value)
      parent_node.appendChild(child_node)

    if child_node.hasChildNodes():      # is there a value already set?
      old_value_tn = child_node.childNodes[0]
      child_node.replaceChild(value_tn, old_value_tn)
      old_value_tn.unlink()             # free memory
    else:
      child_node.appendChild(value_tn)

    return child_node

  ### Return a simple dictionary of data stored as childs of node
  ### FIXME: This routine is a quick hack, and should be generalized in future
  ###        to support more general form of data...
  def get_data_as_dictionary(self, node):
    return_data = {}
    for data in node.childNodes:
      if data.nodeType == Node.ELEMENT_NODE:
	if data.hasChildNodes():
	  value = ""
	  value_node = data.childNodes[0]
	  if value_node.nodeType == Node.TEXT_NODE:
	    value = value_node.nodeValue
	  elif value_node.nodeType == Node.ELEMENT_NODE:
	    value = value_node.localName
     	  return_data[data.localName] = value

    return return_data

  def set_tlm_mesh(self, mesh, bdist = 10):
    units = self.get_units()
    units_length = units[consts.XML_TAG_UNITS_LENGTH]

    dl = mesh.get_dl()/units_length
    bbox = mesh.get_bbox()
    print "Project::set_tlm_mesh, min_dl = ", dl, "; bbox = ", bbox

    # Quick&Dirty hack to come away from the boundary
    badd = bdist*dl

    mesh_node = self.get_node(consts.XML_TAG_MESH)
    self.set_value(mesh_node, consts.XML_TAG_X1, bbox[0][0] - badd)
    self.set_value(mesh_node, consts.XML_TAG_Y1, bbox[0][1] - badd)
    self.set_value(mesh_node, consts.XML_TAG_Z1, bbox[0][2] - badd)
    self.set_value(mesh_node, consts.XML_TAG_X2, bbox[1][0] + badd)
    self.set_value(mesh_node, consts.XML_TAG_Y2, bbox[1][1] + badd)
    self.set_value(mesh_node, consts.XML_TAG_Z2, bbox[1][2] + badd)
    self.set_value(mesh_node, consts.XML_TAG_DX, dl) 
    self.set_value(mesh_node, consts.XML_TAG_DY, dl) 
    self.set_value(mesh_node, consts.XML_TAG_DZ, dl) 

  def new_file_node(self, file_name):
    new_file = self.dom.createElement(consts.XML_TAG_FILE)
    # TODO: attr_name for the file
    
    file_text = self.dom.createTextNode(file_name)
    new_file.appendChild(file_text)
    return new_file


  ###------------------
  ### update_results
  ###------------------
  def update_results(self, results, ref_results = None):
    """ Update the project tree and the EMM data with the new results """
    self.state = consts.PRJ_STATE_MODIFIED

    print "Project:: Updating results..."
    print results

    if ref_results:
      print ">> reference results:"
      print ref_results

    results_node = self.get_node(consts.XML_TAG_RESULTS)
    # TODO: define "files" in consts.py

    # if we have some results with time-domain signal
    #tdsignals_node = None

    ###
    ### time-domain signals ###
    ###
    # if len(results["tdsignals"]):
    #   tdsignals_node = self.set_value(results_node, consts.XML_TAG_TDSIGNALS, '')

    result_node_simulation = self.get_node_with_attr(consts.XML_TAG_RESULT, consts.XML_ATTR_RESULT_TYPE, consts.RESULT_TYPE_SIMULATION, results_node)	# get the node or create a new one

    for tds in results["tdsignals"]:
      #self.set_value_for_attr(tdsignals_node, consts.XML_TAG_RESULT, (consts.XML_ATTR_NAME, tds.name), '')
      tds_node = self.set_value_for_attr(result_node_simulation, consts.XML_TAG_TDSIGNAL, (consts.XML_ATTR_NAME, tds.name), '')
      file_node = self.set_value_for_attr(tds_node, consts.XML_TAG_FILE, (consts.XML_ATTR_FILE_SRC, tds.file_source), '')
      self.set_attr(file_node, consts.XML_ATTR_FILE_COLUMN, tds.column)

    ###
    ### reference results
    ###
    if ref_results:
      result_node_ref = self.get_node_with_attr(consts.XML_TAG_RESULT, consts.XML_ATTR_RESULT_TYPE, consts.RESULT_TYPE_REFERENCE, results_node)	# get the node or create a new one

      for tds in ref_results["tdsignals"]:
	tds_node = self.set_value_for_attr(result_node_ref, consts.XML_TAG_TDSIGNAL, (consts.XML_ATTR_NAME, tds.name), '')
	file_node = self.set_value_for_attr(tds_node, consts.XML_TAG_FILE, (consts.XML_ATTR_FILE_SRC, tds.file_source), '')
	self.set_attr(file_node, consts.XML_ATTR_FILE_COLUMN, tds.column)


    ###
    ### S-parameters ###
    ###
    # if len(results["sparams"]):
    #   sparams_node = self.set_value(results_node, consts.XML_TAG_SPARAMETERS, '')

    for spar in results["sparams"]:
      for spar_name in spar.get_names():
	#self.set_value_for_attr(sparams_node, consts.XML_TAG_RESULT, (consts.XML_ATTR_NAME, spar_name), '')
	self.set_value_for_attr(results_node, consts.XML_TAG_SPARAMETERS, (consts.XML_ATTR_NAME, spar_name), '')

    ###
    ### Impedance ###
    ###
    for imp in results["zparams"]:
      for zpar_name in imp.get_names():
	self.set_value_for_attr(results_node, consts.XML_TAG_ZPARAMETERS, (consts.XML_ATTR_NAME, zpar_name), '')

    for file in results["files"]:
      results_node.appendChild(self.new_file_node(file))

    for res in results["resobjs"]:
      self.set_value_for_attr(results_node, consts.XML_TAG_RESULT, (consts.XML_ATTR_NAME, res.name), '')

  def close(self):
    self.dom.unlink()		# destroy the DOM object
    del self.dom
    self.dom = None
    self.fname = consts.PRJ_DEFAULT_FNAME
    ## FIXME: the next line was setting state to PRJ_STATE_INITIALIZED, which was used
    ##         in tests in other methods of this class, but has not initiated to the main
    ##         window, that the project was saved; Think of where to set the state of the project
    ##         to PRJ_STATE_INITIALIZED again!
    self.state = consts.PRJ_STATE_INITIALIZED


###############################################
if __name__ == '__main__':
  print "Testing Project Module..."

  prj = Project()
  #prj.new(["TestModule", "1e-3"])



#######################################################################
#
# $Id: lsBaseDialog1.py,v 1.2 2008-02-07 05:45:25 lorenz Exp $
# 
# Copyright (C) 2004-2008 Petr Lorenz
# All rights reserved.
# 
# ##### BEGIN LICENSE BLOCK #####
#
# Version: MPL 1.1 / GPL 2.0 / LGPL 2.1
#
# The contents of this file are subject to the Mozilla Public License
# Version 1.1 (the "License"); you may not use this file except in
# compliance with the License. You may obtain a copy of the License at
# http://www.mozilla.org/MPL/
# 
# Software distributed under the License is distributed on an "AS IS"
# basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
# License for the specific language governing rights and limitations
# under the License.
# 
# The Original Code is Petr Lorenz emGine Graphical User Interface (GUI).
# 
# The Initial Developer of the Original Code is Petr Lorenz.
# Portions created by the Initial Developer are Copyright (C) 2004-2008.
# All Rights Reserved.
# 
# Alternatively, the contents of this file may be used under the terms of
# either the GNU General Public License Version 2 or later (the  "GPL"), or the
# GNU Lesser General Public License Version 2.1 or later (the "LGPL"), in which
# case the provisions of the GPL or the LGPL are applicable instead of those
# above.  If you wish to allow use of your version of this file only under the
# terms of either the GPL or the LGPL and not to allow others to use your
# version of this file under the MPL, indicate your decision by deleting the
# provisions above and replace them with the notice and other provisions
# required by the GPL or LGPL.  If you do not delete the provisions above, a
# recipient may use your version of this file under the terms of any one of the
# MPL License, the GPL or the LGPL.
#
# ##### END LICENSE BLOCK #####
#
#######################################################################
#
# $Log: lsBaseDialog1.py,v $
# Revision 1.2  2008-02-07 05:45:25  lorenz
# * Fixed 'Simulation Boundary View' check problem
#   when closing a project.
#
# * Fixed Bounding Box problems when moving/editing a geometry.
#
# Revision 1.1.1.1  2008-01-28 06:15:35  lorenz
# Import emGine GUI v. 0.7.1
#
#######################################################################

# July 2007 - January 2008

# abstract class of base dialog 1

import wx

from emGineValidators import UniqueNameValidator

class lsBaseDialog1(wx.Dialog):
  def __init__(self, title):
    wx.Dialog.__init__(self, None, -1, title) #, pos = (0,0))
    self.sizer = None
    self.box = None
    self.sb_sizer = None	# sizer for static box

    # initialize list of entries containing user data objects
    self.text_entries = None
    self.item_entries = list()
    self.items_lists = list()
    self.materials_choice = None

    # Show choice of materials
    self.show_materials = False
    self.materials = None
    self.mat_list = None
    self.mat_choices_data_index = None
    self.sel_material = None		# material selected per default

    # Store name entry widget
    self.name_entry = None

  # overridden method
  def ShowModal(self):
    sizer = self.sizer

    if self.show_materials:
      mat_choices = None
      if self.mat_list:
	mat_choices_list = self.mat_list
      else:
	# build the list of material names
	self.mat_list = mat_choices_list = self.materials.get_list_of_names()

      materials_text = wx.StaticText(self, -1, "Material:")
      self.materials_choice = materials_choice = wx.Choice(self, -1, (150, -1), choices = mat_choices_list)
      choice = 0
      if self.sel_material:
	choice = self.materials.get_index_by_name(self.sel_material.name)
	if choice == None:
	  choice = 0
      materials_choice.SetSelection(choice)

      #box_mat = wx.StaticBox(self, -1, "Material choice")
      #sb_sizer_mat = wx.StaticBoxSizer(box_mat, wx.VERTICAL)

      mat_sizer = wx.FlexGridSizer(1, 3, 10, 6)	# rows, cols, hgap, vgap
      mat_sizer.Add((10,0), 1)
      mat_sizer.Add(materials_text, 0, wx.ALIGN_RIGHT | wx.ALIGN_CENTER_VERTICAL)
      mat_sizer.Add(materials_choice, 0, wx.ALIGN_RIGHT | wx.ALIGN_CENTER_VERTICAL)

      #sb_sizer_mat.Add(mat_sizer, 0, wx.EXPAND | wx.ALL, 5)

      sizer.Add(mat_sizer, 0, wx.ALIGN_LEFT | wx.ALIGN_CENTER_VERTICAL, 5)
      #sizer.Add(sb_sizer_mat, 0, wx.ALIGN_LEFT | wx.ALIGN_CENTER_VERTICAL, 5)

    # Realize OK/Cancel buttons
    ok_butt = wx.Button(self, wx.ID_OK)
    ok_butt.SetDefault()
    cancel = wx.Button(self, wx.ID_CANCEL)

    btns = wx.StdDialogButtonSizer()
    btns.AddButton(ok_butt)
    btns.AddButton(cancel)
    btns.Realize()

    sizer.Add(btns, 0, wx.EXPAND | wx.ALL, 5)

    self.SetSizer(sizer)
    sizer.Fit(self)

    return wx.Dialog.ShowModal(self)

  def create_static_box(self, title):
    self.box = wx.StaticBox(self, -1, title)
    self.sb_sizer = wx.StaticBoxSizer(self.box, wx.VERTICAL)

  def set_user_sizer(self, user_sizer):
   #sizer = 
    if (self.sizer == None):
      sizer = self.sizer = wx.BoxSizer(wx.VERTICAL)
      if (self.sb_sizer):
	self.sb_sizer.Add(user_sizer, 0, wx.EXPAND | wx.ALL, 5)
	sizer.Add(self.sb_sizer, 0, wx.EXPAND | wx.ALL, 5)
      else:
	sizer.Add(user_sizer, 0, wx.EXPAND | wx.ALL, 5)

  ### Name checking
  def set_name_entry(self, entry_widget):
    self.name_entry = entry_widget

  def turn_on_unique_name_checking(self, used_names = None):
    if not self.name_entry:
      return False

    self.name_entry.SetValidator(UniqueNameValidator(used_names))
    return True

  ### Materials
  def set_materials(self, materials):
    self.show_materials = True
    self.materials = materials

  def set_selected_material(self, sel_material):
    self.sel_material = sel_material

  def set_material_list(self, mat_list):
    self.mat_list = mat_list

  ### Data
  def set_data_text_fields(self, text_entries):
    #print "Setting text fields data..."
    self.text_entries = text_entries

  def set_data_items(self, item_entries):
    self.item_entries = item_entries

  # return the index of the new item
  def add_data_items(self, item_entries):
    if isinstance(self.item_entries, list):
      self.item_entries.append(item_entries)
      return len(self.item_entries)-1
    return None

  def set_items_lists(self, items_lists):
    self.items_lists = items_lists

  # return the index of the new item
  def add_items_lists(self, items_lists):
    if isinstance(self.items_lists, list):
      self.items_lists.append(items_lists)
      return len(self.items_lists)-1
    return None

  def set_data_grid_plain(self, grid_entries = []):
    self.grid_entries = grid_entries

  def get_data(self):
    #print "Getting data..."
    data = list()
    if (self.text_entries):
      for entry in self.text_entries:
	data.append(entry.GetValue())

    if (self.item_entries):
      for ind in range(len(self.item_entries)):
	entry = self.item_entries[ind]
	sel_index = entry.GetSelection()
	data.append(self.items_lists[ind][sel_index])

    # last entry is always a chosen material, if material can be chosen
    mat_name = self.get_data_material()
    if mat_name:
      data.append(mat_name)

    return data

  def get_data_material(self):
    if self.materials_choice:
      sel_index = self.materials_choice.GetSelection()
      #print "***** sel_index:\n" + str(sel_index) + "\n"
      return self.mat_list[sel_index]
    return None

  def get_data_grid(self):
    datas = list()
    for grid in self.grid_entries:
      data = list()
      cols = grid.GetNumberCols()
      rows = grid.GetNumberRows()
      for r_ind in range(rows):
	r_entry = list()
	for c_ind in range(cols):
	  val = grid.GetCellValue(r_ind, c_ind) 
	  if val:
	    r_entry.append(val)
	data.append(r_entry)
      datas.append(data)
    return datas




#######################################################################
#
# $Id: Plot2D_vtk.py,v 1.2 2008-02-07 05:45:25 lorenz Exp $
# 
# Copyright (C) 2004-2008 Petr Lorenz
# All rights reserved.
# 
# ##### BEGIN LICENSE BLOCK #####
#
# Version: MPL 1.1 / GPL 2.0 / LGPL 2.1
#
# The contents of this file are subject to the Mozilla Public License
# Version 1.1 (the "License"); you may not use this file except in
# compliance with the License. You may obtain a copy of the License at
# http://www.mozilla.org/MPL/
# 
# Software distributed under the License is distributed on an "AS IS"
# basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
# License for the specific language governing rights and limitations
# under the License.
# 
# The Original Code is Petr Lorenz emGine Graphical User Interface (GUI).
# 
# The Initial Developer of the Original Code is Petr Lorenz.
# Portions created by the Initial Developer are Copyright (C) 2004-2008.
# All Rights Reserved.
# 
# Alternatively, the contents of this file may be used under the terms of
# either the GNU General Public License Version 2 or later (the  "GPL"), or the
# GNU Lesser General Public License Version 2.1 or later (the "LGPL"), in which
# case the provisions of the GPL or the LGPL are applicable instead of those
# above.  If you wish to allow use of your version of this file only under the
# terms of either the GPL or the LGPL and not to allow others to use your
# version of this file under the MPL, indicate your decision by deleting the
# provisions above and replace them with the notice and other provisions
# required by the GPL or LGPL.  If you do not delete the provisions above, a
# recipient may use your version of this file under the terms of any one of the
# MPL License, the GPL or the LGPL.
#
# ##### END LICENSE BLOCK #####
#
#######################################################################
# 
# $Log: Plot2D_vtk.py,v $
# Revision 1.2  2008-02-07 05:45:25  lorenz
# * Fixed 'Simulation Boundary View' check problem
#   when closing a project.
#
# * Fixed Bounding Box problems when moving/editing a geometry.
#
# Revision 1.1.1.1  2008-01-28 06:15:35  lorenz
# Import emGine GUI v. 0.7.1
#
# Revision 1.2  2007-07-02 06:37:18  lorenz
# Clean up of old code.
#
# Revision 1.1.1.1  2007-05-08 06:22:35  lorenz
# Initial Import
#
#######################################################################

#
# June-May 2006
#
# 2D plotting with vtk
#
import wx

import vtk
from vtk.wx.wxVTKRenderWindow import wxVTKRenderWindow

import XYDataSource

class Plot2D:
  def __init__(self, master):
    self.polydata = None
    self.x_data = vtk.vtkFloatArray()
    self.y_data = vtk.vtkFloatArray()
    self.hold = 0
    
    # create the VTK-based Plot
    vtk_render_widget = self.vtk_render_widget = wxVTKRenderWindow(master, -1)
    self.hide()		# hide the widget

    xyplot = self.xyplot = vtk.vtkXYPlotActor()
    xyplot.GetProperty().SetColor(1, 0, 0)
    xyplot.GetProperty().SetLineWidth(2)
    xyplot.GetPositionCoordinate().SetValue(0.05, 0.15, 0.0)
    xyplot.GetPosition2Coordinate().SetValue(0.9, 0.7, 1.0)
    xyplot.SetXTitle("")
    xyplot.SetYTitle("")
    tprop = xyplot.GetAxisLabelTextProperty()
    tprop.SetColor(0, 0, 0)
    tprop.ShadowOff()
    tprop.SetFontFamilyToArial()
    #tprop.ItalicOff()
    tprop.ItalicOn()
    tprop.BoldOff()

    ren = self.ren = vtk.vtkRenderer()
    ren.SetBackground(1.00, 1.00, 1.00)
    #ren.SetViewport(0.5, 0.0, 1.0, 1.0)

    self.rw = vtk_render_widget.GetRenderWindow()
    self.rw.AddRenderer(ren) 

  def hold_on(self):
    self.hold = 1

  def hold_off(self):
    self.hold = 0

  def set_dimensions(self, x1, y1, x2, y2):
    self.vtk_render_widget.SetDimensions(x1, y1, x2, y2)

  def show(self):
    self.vtk_render_widget.Show(True)

  def hide(self):
    self.vtk_render_widget.Show(False)

  # Description:
  # Set the floating point values x and y
  def set_data_1(self, xvalues, yvalues):
    if (len(xvalues) != len(yvalues)):
      print "!! Error! Something is wrong with your data to plot"
      return

    #
    # TODO: check if data already set, if so, reallocate (?)
    #
    len_data = len(xvalues)
    self.x_data.SetNumberOfTuples(len_data)
    self.y_data.SetNumberOfTuples(len_data)

    # assign the data
    # TODO: could we use SetArray method directly ?
    for i in range(len_data):
      self.x_data.SetTuple1(i, xvalues[i])
      self.y_data.SetTuple1(i, yvalues[i])


  def plot_data(self, xvalues, yvalues):
    if (self.polydata):
      self.ren.RemoveActor2D(self.xyplot)
      self.xyplot.RemoveInput(self.polydata)
      #return

    if (len(xvalues) != len(yvalues)):
      print "!! Error! Something is wrong with your data to plot"
      return

    len_data = len(xvalues)
    points = vtk.vtkPoints()
    vals = vtk.vtkFloatArray()
    for i in range(len_data):
      points.InsertPoint(i, xvalues[i], 0, 0)
      vals.InsertTuple1(i, yvalues[i])

    self.polydata = polydata = vtk.vtkPolyData()
    polydata.SetPoints(points)
    polydata.GetPointData().SetScalars(vals)

    self.xyplot.AddInput(polydata)
    self.xyplot.SetXValuesToValue()
    self.ren.AddActor2D(self.xyplot)

  def plot_ascii_file(self, fname, column_y = 2, column_x = 1):
    xydata = XYDataSource.XYDataSource()
    xydata.read_from_ascii_file(fname, column_y, column_x)
    self.plot_data(xydata.XYData.get_xlist(), xydata.XYData.get_ylist())

    ## xydata.XYData.fft()
    ## #y_data = xydata.XYData.get_yarray().real.tolist()
    ## y_data = xydata.XYData.get_ylist_abs()
    ## self.plot_data(xydata.XYData.get_xlist(), y_data)

    # TODO: below is the old code, delete it, no more needed
    #f = open(fname)
    #data = f.read().split("\n")
    #x_data = list()
    #y_data = list()
    #for entry in data:
    #  if (entry != ''):
    #	e = entry.split()
    #	x_data.append(float(e[column_x-1])) 
    #	y_data.append(float(e[column_y-1])) 
    #
    #self.plot_data(x_data, y_data)

  def plot_dummy(self):
    x_data = [0.0e-2, 1.0e-2, 2.0e-2, 3.0e-2, 4.0e-2, 5.0e-2, 6.0e-2, 7.0e-2, 8.0e-2, 9.0e-2, 10.0e-2]
    y_data = [0.0, 1.0, 2.0, 3.0, 4.0, 5.0, 4.0, 3.0, 2.0, 1.0, 0.0]
    self.plot_data(x_data, y_data)



###############################################
if __name__ == '__main__':
  print "Testing Plot2D Module..." 


  #### root = Tk()
  #### plot = Plot2D(root)

  #### root.grid_rowconfigure(0, weight = 1)
  #### root.grid_columnconfigure(0, weight = 1) 

  #### plot.plot_dummy()
  #### #plot.plot_data(x_data, y_data)
  #### #plot.plot_ascii_file("EH.tst_")

  #### plot.show()





#######################################################################
#
# $Id: Group.py,v 1.2 2008-02-07 05:45:25 lorenz Exp $
# 
# Copyright (C) 2004-2008 Petr Lorenz
# All rights reserved.
# 
# ##### BEGIN LICENSE BLOCK #####
#
# Version: MPL 1.1 / GPL 2.0 / LGPL 2.1
#
# The contents of this file are subject to the Mozilla Public License
# Version 1.1 (the "License"); you may not use this file except in
# compliance with the License. You may obtain a copy of the License at
# http://www.mozilla.org/MPL/
# 
# Software distributed under the License is distributed on an "AS IS"
# basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
# License for the specific language governing rights and limitations
# under the License.
# 
# The Original Code is Petr Lorenz emGine Graphical User Interface (GUI).
# 
# The Initial Developer of the Original Code is Petr Lorenz.
# Portions created by the Initial Developer are Copyright (C) 2004-2008.
# All Rights Reserved.
# 
# Alternatively, the contents of this file may be used under the terms of
# either the GNU General Public License Version 2 or later (the  "GPL"), or the
# GNU Lesser General Public License Version 2.1 or later (the "LGPL"), in which
# case the provisions of the GPL or the LGPL are applicable instead of those
# above.  If you wish to allow use of your version of this file only under the
# terms of either the GPL or the LGPL and not to allow others to use your
# version of this file under the MPL, indicate your decision by deleting the
# provisions above and replace them with the notice and other provisions
# required by the GPL or LGPL.  If you do not delete the provisions above, a
# recipient may use your version of this file under the terms of any one of the
# MPL License, the GPL or the LGPL.
#
# ##### END LICENSE BLOCK #####
#
#######################################################################
#
# $Log: Group.py,v $
# Revision 1.2  2008-02-07 05:45:25  lorenz
# * Fixed 'Simulation Boundary View' check problem
#   when closing a project.
#
# * Fixed Bounding Box problems when moving/editing a geometry.
#
# Revision 1.1.1.1  2008-01-28 06:15:35  lorenz
# Import emGine GUI v. 0.7.1
#
# Revision 1.3  2007-07-02 06:37:17  lorenz
# Clean up of old code.
#
# Revision 1.2  2007-06-14 06:31:51  lorenz
# Added emGine settings handling.
# Better handling of GUI state through GUIState class.
#
# Revision 1.1.1.1  2007-05-08 06:22:35  lorenz
# Initial Import
#
#######################################################################

# July 2006 - January 2008

# abstract class

from tlmObject import tlmObject

class Group(tlmObject):
  def __init__(self, type_ = "Group"):
    tlmObject.__init__(self, type_)
    self.group_list = list()

  def add(self, new_object):
    self.group_list.append(new_object)

  def get_list(self):
    return self.group_list

  def get_object_by_name(self, objname):
    for obj in self.group_list:
      if obj.name == objname:
        return obj
    return None

  def get_list_of_names(self):
    names = list()
    for obj in self.group_list:
      names.append(obj.name)
    return names

  def get_index_by_name(self, objname):
    obj = self.get_object_by_name(objname)
    if not obj:
      return None
    return self.group_list.index(obj)

  def replace_object(self, old_object, new_object):
    ind = self.group_list.index(old_object)
    self.group_list[ind] = new_object

  def get_info(self):
    info = "Group info:\n"
    info += " Number of objects: " + str(len(self.group_list)) + "\n"
    info += " group_list:\n" + str(self.group_list) + "\n"
    return info




#######################################################################
#
# $Id: ResultSParameters.py,v 1.2 2008-02-07 05:45:25 lorenz Exp $
# 
# Copyright (C) 2004-2008 Petr Lorenz
# All rights reserved.
# 
# ##### BEGIN LICENSE BLOCK #####
#
# Version: MPL 1.1 / GPL 2.0 / LGPL 2.1
#
# The contents of this file are subject to the Mozilla Public License
# Version 1.1 (the "License"); you may not use this file except in
# compliance with the License. You may obtain a copy of the License at
# http://www.mozilla.org/MPL/
# 
# Software distributed under the License is distributed on an "AS IS"
# basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
# License for the specific language governing rights and limitations
# under the License.
# 
# The Original Code is Petr Lorenz emGine Graphical User Interface (GUI).
# 
# The Initial Developer of the Original Code is Petr Lorenz.
# Portions created by the Initial Developer are Copyright (C) 2004-2008.
# All Rights Reserved.
# 
# Alternatively, the contents of this file may be used under the terms of
# either the GNU General Public License Version 2 or later (the  "GPL"), or the
# GNU Lesser General Public License Version 2.1 or later (the "LGPL"), in which
# case the provisions of the GPL or the LGPL are applicable instead of those
# above.  If you wish to allow use of your version of this file only under the
# terms of either the GPL or the LGPL and not to allow others to use your
# version of this file under the MPL, indicate your decision by deleting the
# provisions above and replace them with the notice and other provisions
# required by the GPL or LGPL.  If you do not delete the provisions above, a
# recipient may use your version of this file under the terms of any one of the
# MPL License, the GPL or the LGPL.
#
# ##### END LICENSE BLOCK #####
#
#######################################################################
# 
# $Log: ResultSParameters.py,v $
# Revision 1.2  2008-02-07 05:45:25  lorenz
# * Fixed 'Simulation Boundary View' check problem
#   when closing a project.
#
# * Fixed Bounding Box problems when moving/editing a geometry.
#
# Revision 1.1.1.1  2008-01-28 06:15:35  lorenz
# Import emGine GUI v. 0.7.1
#
# Revision 1.4  2007-07-02 06:37:18  lorenz
# Clean up of old code.
#
# Revision 1.3  2007-06-22 04:07:19  lorenz
# Calculation of dt (delta t) from first column of TDSignal.
#
# Small changes in the computation of S-Parameters.
#
# Revision 1.2  2007-05-14 04:53:51  lorenz
# New code for multiport time-domain signals and S-parameters
#
# Revision 1.1.1.1  2007-05-08 06:22:35  lorenz
# Initial Import
#
#######################################################################

# June 2006 - November 2007

from Result import Result

from XYData import XYData
from FFTData import FFTData

import numpy
import math

#from ResultImpedance import ResultImpedance

# TODO: There are more ways how to compute S-Parameters from TD signals.
#       The variable self.type contains the information about how 
#       S-Parameters are obtained. Document that better.
#
# reference_data_ -- reference simulation results (usualy broadband)
#
class ResultSParameters(Result):
  def __init__(self, input_data_, reference_data_ = None, name_ = 'SParameters'):
    Result.__init__(self, name_)
    self.type = "NotSpecified"
    self.sparameters = XYData()
    self.ref_impedance = 50
    self.tdsignal = None
    self.ref_tdsignals = None
    self.number_of_ports = 1
    self.excited_ports = [1]

    self.fmin = 1e8		# default minimum frequency
    self.fmax = 1e9		# default maximum frequency
    self.freq_points = 201	# default number of points
    self.dt = float(1/1e12)	# default time-step

    if isinstance(input_data_, list):
      # if input_data_ is a list then the S-Parameters
      # contain tdsignals for multiple ports
      print "Computing Multiport S-Parameters"
      self.type = "FromTDSignal"
      self.number_of_ports = len(input_data_)
      self.tdsignal = input_data_
      if reference_data_:
	self.ref_tdsignals = reference_data_

    elif isinstance(input_data_, XYData):
      print "SParameters are given as data, good!"
      self.impedance = None
      self.type = "FromTDSignal"
      self.tdsignal = input_data_
    else:
      self.impedance = input_data_
      self.type = "FromImpedance"

  def set_fmin_fmax(self, fmin_, fmax_):
    self.fmin = float(fmin_)
    self.fmax = float(fmax_)

  def set_dt(self, dt_):
    self.dt = dt_

  def set_excited_ports(self, excited_ports_):
    self.excited_ports = excited_ports_

  def update(self):
    if self.type == "FromTDSignal":
      print "computing S-Parameters from TD Signals"

      # first compute the frequency axis
      df = float((self.fmax - self.fmin)/self.freq_points)
      N = round(1/(df*self.dt))
      N_fft = int(2**(math.ceil(math.log(N)/math.log(2))))
      df_new = 1/(N_fft*self.dt)
      Nmin = int(math.floor(self.fmin/df_new)) 
      Nmax = int(math.ceil(self.fmax/df_new))
      self.freq_points = int(Nmax - Nmin + 1)

      #Nmin = 1		# FIXME: This is now hard-coded, no minimum frequency considered
      self.sparameters.set_xvalues( df_new*(numpy.arange(Nmin, Nmax+1)) )

      print "fmin [GHz] =", self.fmin/1e9
      print "fmax [GHz] =", self.fmax/1e9
      print "fmin_new [GHz] =", Nmin*df_new/1e9
      print "fmax_new [GHz] =", Nmax*df_new/1e9
      print "freq_points =", self.freq_points
      print "df [GHz] =", df/1e9
      print "dt [ns] = ", self.dt/1e-9 
      print "df_new [GHz] =", df_new/1e9
      print "N_fft = ", N_fft
      print "Nmin = ", Nmin
      print "Nmax = ", Nmax

      if (self.number_of_ports > 1):
	#
	# s-parameters for multiple ports #
	#
	# FIXME: quick hack to get basic functionality
	if ( len(self.excited_ports) == 1 ):
	  if 1 in self.excited_ports:
	    print "updating S-parameters for two ports but just with 1 port excited"

	    s11 = None
	    s21 = None

	    if self.ref_tdsignals:
	      fftop = FFTData()

	      print ">> using reference data for the computation of S-parameters <<"

	      #e_ref = self.ref_tdsignals[0].get_yarray() + self.ref_tdsignals[1].get_yarray()
	      h_ref = self.ref_tdsignals[0].get_yarray() - self.ref_tdsignals[1].get_yarray()

	      #e_tot1 = self.tdsignal[0].get_yarray() + self.tdsignal[1].get_yarray()
	      h_tot1 = self.tdsignal[0].get_yarray() - self.tdsignal[1].get_yarray()

	      #e_tot2 = self.tdsignal[2].get_yarray() + self.tdsignal[3].get_yarray()
	      h_tot2 = self.tdsignal[2].get_yarray() - self.tdsignal[3].get_yarray()

	      #E_ref = fftop.array_to_array(e_ref, N_fft)
	      H_ref = fftop.array_to_array(h_ref, N_fft)

	      #E_tot1 = fftop.array_to_array(e_tot1, N_fft)
	      H_tot1 = fftop.array_to_array(h_tot1, N_fft)

	      #E_tot2 = fftop.array_to_array(e_tot2, N_fft)
	      H_tot2 = fftop.array_to_array(h_tot2, N_fft)

	      s11 = (H_ref - H_tot1) / H_ref
	      s21 = H_tot2 / H_ref

	    else:
	      # self.sparameters.add_yvalues(self.tdsignal[0].get_yarray())
	      # self.sparameters.add_yvalues(self.tdsignal[1].get_yarray())
	      # self.sparameters.fft_all(N_fft)
	      # s11 = self.sparameters.get_yarray(0)
	      # s21 = self.sparameters.get_yarray(1)
	      None

	    # print s11
	    # print s21
	    # print len(s11)
	    # print len(s21)
	    # print "----------"

	    ## self.sparameters.set_yvalues(s11[Nmin-2:Nmax-1], 0)
	    ## self.sparameters.set_yvalues(s21[Nmin-2:Nmax-1], 1)
	    self.sparameters.set_yvalues(s11[Nmin:Nmax+1])	# elem 0
	    self.sparameters.add_yvalues(s21[Nmin:Nmax+1])	# elem 1
	    self.names = ["s11", "s21"]
      else:
	#
	# just one port s-parameters #
	#
	#self.sparameters.set_yvalues(df*numpy.arange(1, self.freq_points+1))	# just for testing
	self.sparameters.set_yvalues(self.tdsignal.get_yarray())
	self.sparameters.fft(N_fft)
	y_vals = self.sparameters.get_yarray()
	## if Nmin > 1:
	##   Nmin = Nmin - 2
	self.sparameters.set_yvalues(y_vals[Nmin:Nmax+1])
	self.names = ["s11"]
	print "len(y_vals) = ", len(y_vals)
	print "len(self.sparameters.get_yarray()) = ", len(self.sparameters.get_yarray())

    elif self.type == "FromImpedance":
      imp_data = self.impedance.get_data()
      self.sparameters.set_xvalues(imp_data.get_xarray())
      ZL = imp_data.get_yarray()
      Z0 = self.ref_impedance
      self.sparameters.add_yvalues( (ZL - Z0)/(ZL + Z0) )
      self.names = ["s11"]
    
  def get_data(self, name = None):
    return self.sparameters

###############################################
if __name__ == '__main__':
  print "Testing ResultSParameters Module..."
